# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SCENE_FILE_MERGE_METADATA = Metadata(
    id="b75bdc86e379ec346063e577c5b0039d15d9a56a.boutiques",
    name="scene-file-merge",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


SceneFileMergeUpToParameters = typing.TypedDict('SceneFileMergeUpToParameters', {
    "@type": typing.NotRequired[typing.Literal["up_to"]],
    "last_column": str,
    "opt_reverse": bool,
})
SceneFileMergeUpToParametersTagged = typing.TypedDict('SceneFileMergeUpToParametersTagged', {
    "@type": typing.Literal["up_to"],
    "last_column": str,
    "opt_reverse": bool,
})


SceneFileMergeSceneParameters = typing.TypedDict('SceneFileMergeSceneParameters', {
    "@type": typing.NotRequired[typing.Literal["scene"]],
    "scene": str,
    "up_to": typing.NotRequired[SceneFileMergeUpToParameters | None],
})
SceneFileMergeSceneParametersTagged = typing.TypedDict('SceneFileMergeSceneParametersTagged', {
    "@type": typing.Literal["scene"],
    "scene": str,
    "up_to": typing.NotRequired[SceneFileMergeUpToParameters | None],
})


SceneFileMergeSceneFileParameters = typing.TypedDict('SceneFileMergeSceneFileParameters', {
    "@type": typing.NotRequired[typing.Literal["scene_file"]],
    "scene_file": str,
    "scene": typing.NotRequired[list[SceneFileMergeSceneParameters] | None],
})
SceneFileMergeSceneFileParametersTagged = typing.TypedDict('SceneFileMergeSceneFileParametersTagged', {
    "@type": typing.Literal["scene_file"],
    "scene_file": str,
    "scene": typing.NotRequired[list[SceneFileMergeSceneParameters] | None],
})


SceneFileMergeParameters = typing.TypedDict('SceneFileMergeParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/scene-file-merge"]],
    "scene_file_out": str,
    "scene_file": typing.NotRequired[list[SceneFileMergeSceneFileParameters] | None],
})
SceneFileMergeParametersTagged = typing.TypedDict('SceneFileMergeParametersTagged', {
    "@type": typing.Literal["workbench/scene-file-merge"],
    "scene_file_out": str,
    "scene_file": typing.NotRequired[list[SceneFileMergeSceneFileParameters] | None],
})


def scene_file_merge_up_to_params(
    last_column: str,
    opt_reverse: bool = False,
) -> SceneFileMergeUpToParametersTagged:
    """
    Build parameters.
    
    Args:
        last_column: the number or name of the last scene to include.
        opt_reverse: use the range in reverse order.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "up_to",
        "last_column": last_column,
        "opt_reverse": opt_reverse,
    }
    return params


def scene_file_merge_up_to_cargs(
    params: SceneFileMergeUpToParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-up-to")
    cargs.append(params.get("last_column", None))
    if params.get("opt_reverse", False):
        cargs.append("-reverse")
    return cargs


def scene_file_merge_scene_params(
    scene: str,
    up_to: SceneFileMergeUpToParameters | None = None,
) -> SceneFileMergeSceneParametersTagged:
    """
    Build parameters.
    
    Args:
        scene: the scene number or name.
        up_to: use an inclusive range of scenes.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "scene",
        "scene": scene,
    }
    if up_to is not None:
        params["up_to"] = up_to
    return params


def scene_file_merge_scene_cargs(
    params: SceneFileMergeSceneParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-scene")
    cargs.append(params.get("scene", None))
    if params.get("up_to", None) is not None:
        cargs.extend(scene_file_merge_up_to_cargs(params.get("up_to", None), execution))
    return cargs


def scene_file_merge_scene_file_params(
    scene_file: str,
    scene: list[SceneFileMergeSceneParameters] | None = None,
) -> SceneFileMergeSceneFileParametersTagged:
    """
    Build parameters.
    
    Args:
        scene_file: the input scene file.
        scene: specify a scene to use.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "scene_file",
        "scene_file": scene_file,
    }
    if scene is not None:
        params["scene"] = scene
    return params


def scene_file_merge_scene_file_cargs(
    params: SceneFileMergeSceneFileParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-scene-file")
    cargs.append(params.get("scene_file", None))
    if params.get("scene", None) is not None:
        cargs.extend([a for c in [scene_file_merge_scene_cargs(s, execution) for s in params.get("scene", None)] for a in c])
    return cargs


class SceneFileMergeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SceneFileMergeParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def scene_file_merge_params(
    scene_file_out: str,
    scene_file: list[SceneFileMergeSceneFileParameters] | None = None,
) -> SceneFileMergeParametersTagged:
    """
    Build parameters.
    
    Args:
        scene_file_out: output - the output scene file.
        scene_file: specify a scene file to use scenes from.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/scene-file-merge",
        "scene_file_out": scene_file_out,
    }
    if scene_file is not None:
        params["scene_file"] = scene_file
    return params


def scene_file_merge_cargs(
    params: SceneFileMergeParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-scene-file-merge")
    cargs.append(params.get("scene_file_out", None))
    if params.get("scene_file", None) is not None:
        cargs.extend([a for c in [scene_file_merge_scene_file_cargs(s, execution) for s in params.get("scene_file", None)] for a in c])
    return cargs


def scene_file_merge_outputs(
    params: SceneFileMergeParameters,
    execution: Execution,
) -> SceneFileMergeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SceneFileMergeOutputs(
        root=execution.output_file("."),
    )
    return ret


def scene_file_merge_execute(
    params: SceneFileMergeParameters,
    runner: Runner | None = None,
) -> SceneFileMergeOutputs:
    """
    scene-file-merge
    
    Rearrange scenes into a new file.
    
    Takes one or more scene files and constructs a new scene file by
    concatenating specified scenes from them.
    
    Example: wb_command -scene-file-merge out.scene -scene-file first.scene
    -scene 1 -scene-file second.scene
    
    This example would take the first scene from first.scene, followed by all
    scenes from second.scene, and write these scenes to out.scene.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SceneFileMergeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SCENE_FILE_MERGE_METADATA)
    params = execution.params(params)
    cargs = scene_file_merge_cargs(params, execution)
    ret = scene_file_merge_outputs(params, execution)
    execution.run(cargs)
    return ret


def scene_file_merge(
    scene_file_out: str,
    scene_file: list[SceneFileMergeSceneFileParameters] | None = None,
    runner: Runner | None = None,
) -> SceneFileMergeOutputs:
    """
    scene-file-merge
    
    Rearrange scenes into a new file.
    
    Takes one or more scene files and constructs a new scene file by
    concatenating specified scenes from them.
    
    Example: wb_command -scene-file-merge out.scene -scene-file first.scene
    -scene 1 -scene-file second.scene
    
    This example would take the first scene from first.scene, followed by all
    scenes from second.scene, and write these scenes to out.scene.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        scene_file_out: output - the output scene file.
        scene_file: specify a scene file to use scenes from.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SceneFileMergeOutputs`).
    """
    params = scene_file_merge_params(
        scene_file_out=scene_file_out,
        scene_file=scene_file,
    )
    return scene_file_merge_execute(params, runner)


__all__ = [
    "SCENE_FILE_MERGE_METADATA",
    "SceneFileMergeOutputs",
    "scene_file_merge",
    "scene_file_merge_execute",
    "scene_file_merge_params",
    "scene_file_merge_scene_file_params",
    "scene_file_merge_scene_params",
    "scene_file_merge_up_to_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_GRADIENT_METADATA = Metadata(
    id="394970baf2c81170ff2934ce439704993af8638b.boutiques",
    name="metric-gradient",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


MetricGradientPresmoothParameters = typing.TypedDict('MetricGradientPresmoothParameters', {
    "@type": typing.NotRequired[typing.Literal["presmooth"]],
    "kernel": float,
    "opt_fwhm": bool,
})
MetricGradientPresmoothParametersTagged = typing.TypedDict('MetricGradientPresmoothParametersTagged', {
    "@type": typing.Literal["presmooth"],
    "kernel": float,
    "opt_fwhm": bool,
})


MetricGradientRoiParameters = typing.TypedDict('MetricGradientRoiParameters', {
    "@type": typing.NotRequired[typing.Literal["roi"]],
    "roi_metric": InputPathType,
    "opt_match_columns": bool,
})
MetricGradientRoiParametersTagged = typing.TypedDict('MetricGradientRoiParametersTagged', {
    "@type": typing.Literal["roi"],
    "roi_metric": InputPathType,
    "opt_match_columns": bool,
})


MetricGradientParameters = typing.TypedDict('MetricGradientParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/metric-gradient"]],
    "surface": InputPathType,
    "metric_in": InputPathType,
    "metric_out": str,
    "presmooth": typing.NotRequired[MetricGradientPresmoothParameters | None],
    "roi": typing.NotRequired[MetricGradientRoiParameters | None],
    "opt_vectors_vector_metric_out": typing.NotRequired[str | None],
    "opt_column_column": typing.NotRequired[str | None],
    "opt_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
    "opt_average_normals": bool,
})
MetricGradientParametersTagged = typing.TypedDict('MetricGradientParametersTagged', {
    "@type": typing.Literal["workbench/metric-gradient"],
    "surface": InputPathType,
    "metric_in": InputPathType,
    "metric_out": str,
    "presmooth": typing.NotRequired[MetricGradientPresmoothParameters | None],
    "roi": typing.NotRequired[MetricGradientRoiParameters | None],
    "opt_vectors_vector_metric_out": typing.NotRequired[str | None],
    "opt_column_column": typing.NotRequired[str | None],
    "opt_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
    "opt_average_normals": bool,
})


def metric_gradient_presmooth_params(
    kernel: float,
    opt_fwhm: bool = False,
) -> MetricGradientPresmoothParametersTagged:
    """
    Build parameters.
    
    Args:
        kernel: the size of the gaussian smoothing kernel in mm, as sigma by\
            default.
        opt_fwhm: kernel size is FWHM, not sigma.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "presmooth",
        "kernel": kernel,
        "opt_fwhm": opt_fwhm,
    }
    return params


def metric_gradient_presmooth_cargs(
    params: MetricGradientPresmoothParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-presmooth")
    cargs.append(str(params.get("kernel", None)))
    if params.get("opt_fwhm", False):
        cargs.append("-fwhm")
    return cargs


def metric_gradient_roi_params(
    roi_metric: InputPathType,
    opt_match_columns: bool = False,
) -> MetricGradientRoiParametersTagged:
    """
    Build parameters.
    
    Args:
        roi_metric: the area to take the gradient within, as a metric.
        opt_match_columns: for each input column, use the corresponding column\
            from the roi.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "roi",
        "roi_metric": roi_metric,
        "opt_match_columns": opt_match_columns,
    }
    return params


def metric_gradient_roi_cargs(
    params: MetricGradientRoiParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-roi")
    cargs.append(execution.input_file(params.get("roi_metric", None)))
    if params.get("opt_match_columns", False):
        cargs.append("-match-columns")
    return cargs


class MetricGradientOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricGradientParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the magnitude of the gradient"""
    opt_vectors_vector_metric_out: OutputPathType | None
    """output gradient vectors: the vectors as a metric file"""


def metric_gradient_params(
    surface: InputPathType,
    metric_in: InputPathType,
    metric_out: str,
    presmooth: MetricGradientPresmoothParameters | None = None,
    roi: MetricGradientRoiParameters | None = None,
    opt_vectors_vector_metric_out: str | None = None,
    opt_column_column: str | None = None,
    opt_corrected_areas_area_metric: InputPathType | None = None,
    opt_average_normals: bool = False,
) -> MetricGradientParametersTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface to compute the gradient on.
        metric_in: the metric to compute the gradient of.
        metric_out: the magnitude of the gradient.
        presmooth: smooth the metric before computing the gradient.
        roi: select a region of interest to take the gradient of.
        opt_vectors_vector_metric_out: output gradient vectors: the vectors as\
            a metric file.
        opt_column_column: select a single column to compute the gradient of:\
            the column number or name.
        opt_corrected_areas_area_metric: vertex areas to use instead of\
            computing them from the surface: the corrected vertex areas, as a\
            metric.
        opt_average_normals: average the normals of each vertex with its\
            neighbors before using them to compute the gradient.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-gradient",
        "surface": surface,
        "metric_in": metric_in,
        "metric_out": metric_out,
        "opt_average_normals": opt_average_normals,
    }
    if presmooth is not None:
        params["presmooth"] = presmooth
    if roi is not None:
        params["roi"] = roi
    if opt_vectors_vector_metric_out is not None:
        params["opt_vectors_vector_metric_out"] = opt_vectors_vector_metric_out
    if opt_column_column is not None:
        params["opt_column_column"] = opt_column_column
    if opt_corrected_areas_area_metric is not None:
        params["opt_corrected_areas_area_metric"] = opt_corrected_areas_area_metric
    return params


def metric_gradient_cargs(
    params: MetricGradientParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-metric-gradient")
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(execution.input_file(params.get("metric_in", None)))
    cargs.append(params.get("metric_out", None))
    if params.get("presmooth", None) is not None:
        cargs.extend(metric_gradient_presmooth_cargs(params.get("presmooth", None), execution))
    if params.get("roi", None) is not None:
        cargs.extend(metric_gradient_roi_cargs(params.get("roi", None), execution))
    if params.get("opt_vectors_vector_metric_out", None) is not None:
        cargs.extend([
            "-vectors",
            params.get("opt_vectors_vector_metric_out", None)
        ])
    if params.get("opt_column_column", None) is not None:
        cargs.extend([
            "-column",
            params.get("opt_column_column", None)
        ])
    if params.get("opt_corrected_areas_area_metric", None) is not None:
        cargs.extend([
            "-corrected-areas",
            execution.input_file(params.get("opt_corrected_areas_area_metric", None))
        ])
    if params.get("opt_average_normals", False):
        cargs.append("-average-normals")
    return cargs


def metric_gradient_outputs(
    params: MetricGradientParameters,
    execution: Execution,
) -> MetricGradientOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricGradientOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric_out", None)),
        opt_vectors_vector_metric_out=execution.output_file(params.get("opt_vectors_vector_metric_out", None)) if (params.get("opt_vectors_vector_metric_out") is not None) else None,
    )
    return ret


def metric_gradient_execute(
    params: MetricGradientParameters,
    runner: Runner | None = None,
) -> MetricGradientOutputs:
    """
    metric-gradient
    
    Surface gradient of a metric file.
    
    At each vertex, the immediate neighbors are unfolded onto a plane tangent to
    the surface at the vertex (specifically, perpendicular to the normal). The
    gradient is computed using a regression between the unfolded positions of
    the vertices and their values. The gradient is then given by the slopes of
    the regression, and reconstructed as a 3D gradient vector. By default, takes
    the gradient of all columns, with no presmoothing, across the whole surface,
    without averaging the normals of the surface among neighbors.
    
    When using -corrected-areas, note that it is an approximate correction.
    Doing smoothing on individual surfaces before averaging/gradient is
    preferred, when possible, in order to make use of the original surface
    structure.
    
    Specifying an ROI will restrict the gradient to only use data from where the
    ROI metric is positive, and output zeros anywhere the ROI metric is not
    positive.
    
    By default, the first column of the roi metric is used for all input
    columns. When -match-columns is specified to the -roi option, the input and
    roi metrics must have the same number of columns, and for each input
    column's index, the same column index is used in the roi metric. If the
    -match-columns option to -roi is used while the -column option is also used,
    the number of columns of the roi metric must match the input metric, and it
    will use the roi column with the index of the selected input column.
    
    The vector output metric is organized such that the X, Y, and Z components
    from a single input column are consecutive columns.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricGradientOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_GRADIENT_METADATA)
    params = execution.params(params)
    cargs = metric_gradient_cargs(params, execution)
    ret = metric_gradient_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_gradient(
    surface: InputPathType,
    metric_in: InputPathType,
    metric_out: str,
    presmooth: MetricGradientPresmoothParameters | None = None,
    roi: MetricGradientRoiParameters | None = None,
    opt_vectors_vector_metric_out: str | None = None,
    opt_column_column: str | None = None,
    opt_corrected_areas_area_metric: InputPathType | None = None,
    opt_average_normals: bool = False,
    runner: Runner | None = None,
) -> MetricGradientOutputs:
    """
    metric-gradient
    
    Surface gradient of a metric file.
    
    At each vertex, the immediate neighbors are unfolded onto a plane tangent to
    the surface at the vertex (specifically, perpendicular to the normal). The
    gradient is computed using a regression between the unfolded positions of
    the vertices and their values. The gradient is then given by the slopes of
    the regression, and reconstructed as a 3D gradient vector. By default, takes
    the gradient of all columns, with no presmoothing, across the whole surface,
    without averaging the normals of the surface among neighbors.
    
    When using -corrected-areas, note that it is an approximate correction.
    Doing smoothing on individual surfaces before averaging/gradient is
    preferred, when possible, in order to make use of the original surface
    structure.
    
    Specifying an ROI will restrict the gradient to only use data from where the
    ROI metric is positive, and output zeros anywhere the ROI metric is not
    positive.
    
    By default, the first column of the roi metric is used for all input
    columns. When -match-columns is specified to the -roi option, the input and
    roi metrics must have the same number of columns, and for each input
    column's index, the same column index is used in the roi metric. If the
    -match-columns option to -roi is used while the -column option is also used,
    the number of columns of the roi metric must match the input metric, and it
    will use the roi column with the index of the selected input column.
    
    The vector output metric is organized such that the X, Y, and Z components
    from a single input column are consecutive columns.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        surface: the surface to compute the gradient on.
        metric_in: the metric to compute the gradient of.
        metric_out: the magnitude of the gradient.
        presmooth: smooth the metric before computing the gradient.
        roi: select a region of interest to take the gradient of.
        opt_vectors_vector_metric_out: output gradient vectors: the vectors as\
            a metric file.
        opt_column_column: select a single column to compute the gradient of:\
            the column number or name.
        opt_corrected_areas_area_metric: vertex areas to use instead of\
            computing them from the surface: the corrected vertex areas, as a\
            metric.
        opt_average_normals: average the normals of each vertex with its\
            neighbors before using them to compute the gradient.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricGradientOutputs`).
    """
    params = metric_gradient_params(
        surface=surface,
        metric_in=metric_in,
        metric_out=metric_out,
        presmooth=presmooth,
        roi=roi,
        opt_vectors_vector_metric_out=opt_vectors_vector_metric_out,
        opt_column_column=opt_column_column,
        opt_corrected_areas_area_metric=opt_corrected_areas_area_metric,
        opt_average_normals=opt_average_normals,
    )
    return metric_gradient_execute(params, runner)


__all__ = [
    "METRIC_GRADIENT_METADATA",
    "MetricGradientOutputs",
    "metric_gradient",
    "metric_gradient_execute",
    "metric_gradient_params",
    "metric_gradient_presmooth_params",
    "metric_gradient_roi_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

LABEL_PROBABILITY_METADATA = Metadata(
    id="17883125fa07b84610948fe71508e3f8bf4671e0.boutiques",
    name="label-probability",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


LabelProbabilityParameters = typing.TypedDict('LabelProbabilityParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/label-probability"]],
    "label_maps": InputPathType,
    "probability_metric_out": str,
    "opt_exclude_unlabeled": bool,
})
LabelProbabilityParametersTagged = typing.TypedDict('LabelProbabilityParametersTagged', {
    "@type": typing.Literal["workbench/label-probability"],
    "label_maps": InputPathType,
    "probability_metric_out": str,
    "opt_exclude_unlabeled": bool,
})


class LabelProbabilityOutputs(typing.NamedTuple):
    """
    Output object returned when calling `LabelProbabilityParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    probability_metric_out: OutputPathType
    """the relative frequencies of each label at each vertex"""


def label_probability_params(
    label_maps: InputPathType,
    probability_metric_out: str,
    opt_exclude_unlabeled: bool = False,
) -> LabelProbabilityParametersTagged:
    """
    Build parameters.
    
    Args:
        label_maps: label file containing individual label maps from many\
            subjects.
        probability_metric_out: the relative frequencies of each label at each\
            vertex.
        opt_exclude_unlabeled: don't make a probability map of the unlabeled\
            key.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/label-probability",
        "label_maps": label_maps,
        "probability_metric_out": probability_metric_out,
        "opt_exclude_unlabeled": opt_exclude_unlabeled,
    }
    return params


def label_probability_cargs(
    params: LabelProbabilityParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-label-probability")
    cargs.append(execution.input_file(params.get("label_maps", None)))
    cargs.append(params.get("probability_metric_out", None))
    if params.get("opt_exclude_unlabeled", False):
        cargs.append("-exclude-unlabeled")
    return cargs


def label_probability_outputs(
    params: LabelProbabilityParameters,
    execution: Execution,
) -> LabelProbabilityOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = LabelProbabilityOutputs(
        root=execution.output_file("."),
        probability_metric_out=execution.output_file(params.get("probability_metric_out", None)),
    )
    return ret


def label_probability_execute(
    params: LabelProbabilityParameters,
    runner: Runner | None = None,
) -> LabelProbabilityOutputs:
    """
    label-probability
    
    Find frequency of surface labels.
    
    This command outputs a set of soft ROIs, one for each label in the input,
    where the value is how many of the input maps had that label at that vertex,
    divided by the number of input maps.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelProbabilityOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(LABEL_PROBABILITY_METADATA)
    params = execution.params(params)
    cargs = label_probability_cargs(params, execution)
    ret = label_probability_outputs(params, execution)
    execution.run(cargs)
    return ret


def label_probability(
    label_maps: InputPathType,
    probability_metric_out: str,
    opt_exclude_unlabeled: bool = False,
    runner: Runner | None = None,
) -> LabelProbabilityOutputs:
    """
    label-probability
    
    Find frequency of surface labels.
    
    This command outputs a set of soft ROIs, one for each label in the input,
    where the value is how many of the input maps had that label at that vertex,
    divided by the number of input maps.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        label_maps: label file containing individual label maps from many\
            subjects.
        probability_metric_out: the relative frequencies of each label at each\
            vertex.
        opt_exclude_unlabeled: don't make a probability map of the unlabeled\
            key.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelProbabilityOutputs`).
    """
    params = label_probability_params(
        label_maps=label_maps,
        probability_metric_out=probability_metric_out,
        opt_exclude_unlabeled=opt_exclude_unlabeled,
    )
    return label_probability_execute(params, runner)


__all__ = [
    "LABEL_PROBABILITY_METADATA",
    "LabelProbabilityOutputs",
    "label_probability",
    "label_probability_execute",
    "label_probability_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_FIND_CLUSTERS_METADATA = Metadata(
    id="200fd9774a7a04dcd84899fd90bc8dacd0c76bf0.boutiques",
    name="cifti-find-clusters",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


CiftiFindClustersLeftSurfaceParameters = typing.TypedDict('CiftiFindClustersLeftSurfaceParameters', {
    "@type": typing.NotRequired[typing.Literal["left_surface"]],
    "surface": InputPathType,
    "opt_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})
CiftiFindClustersLeftSurfaceParametersTagged = typing.TypedDict('CiftiFindClustersLeftSurfaceParametersTagged', {
    "@type": typing.Literal["left_surface"],
    "surface": InputPathType,
    "opt_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})


CiftiFindClustersRightSurfaceParameters = typing.TypedDict('CiftiFindClustersRightSurfaceParameters', {
    "@type": typing.NotRequired[typing.Literal["right_surface"]],
    "surface": InputPathType,
    "opt_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})
CiftiFindClustersRightSurfaceParametersTagged = typing.TypedDict('CiftiFindClustersRightSurfaceParametersTagged', {
    "@type": typing.Literal["right_surface"],
    "surface": InputPathType,
    "opt_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})


CiftiFindClustersCerebellumSurfaceParameters = typing.TypedDict('CiftiFindClustersCerebellumSurfaceParameters', {
    "@type": typing.NotRequired[typing.Literal["cerebellum_surface"]],
    "surface": InputPathType,
    "opt_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})
CiftiFindClustersCerebellumSurfaceParametersTagged = typing.TypedDict('CiftiFindClustersCerebellumSurfaceParametersTagged', {
    "@type": typing.Literal["cerebellum_surface"],
    "surface": InputPathType,
    "opt_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})


CiftiFindClustersSizeRatioParameters = typing.TypedDict('CiftiFindClustersSizeRatioParameters', {
    "@type": typing.NotRequired[typing.Literal["size_ratio"]],
    "surface_ratio": float,
    "volume_ratio": float,
})
CiftiFindClustersSizeRatioParametersTagged = typing.TypedDict('CiftiFindClustersSizeRatioParametersTagged', {
    "@type": typing.Literal["size_ratio"],
    "surface_ratio": float,
    "volume_ratio": float,
})


CiftiFindClustersDistanceParameters = typing.TypedDict('CiftiFindClustersDistanceParameters', {
    "@type": typing.NotRequired[typing.Literal["distance"]],
    "surface_distance": float,
    "volume_distance": float,
})
CiftiFindClustersDistanceParametersTagged = typing.TypedDict('CiftiFindClustersDistanceParametersTagged', {
    "@type": typing.Literal["distance"],
    "surface_distance": float,
    "volume_distance": float,
})


CiftiFindClustersParameters = typing.TypedDict('CiftiFindClustersParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/cifti-find-clusters"]],
    "cifti": InputPathType,
    "surface_value_threshold": float,
    "surface_minimum_area": float,
    "volume_value_threshold": float,
    "volume_minimum_size": float,
    "direction": str,
    "cifti_out": str,
    "opt_less_than": bool,
    "left_surface": typing.NotRequired[CiftiFindClustersLeftSurfaceParameters | None],
    "right_surface": typing.NotRequired[CiftiFindClustersRightSurfaceParameters | None],
    "cerebellum_surface": typing.NotRequired[CiftiFindClustersCerebellumSurfaceParameters | None],
    "opt_cifti_roi_roi_cifti": typing.NotRequired[InputPathType | None],
    "opt_merged_volume": bool,
    "size_ratio": typing.NotRequired[CiftiFindClustersSizeRatioParameters | None],
    "distance": typing.NotRequired[CiftiFindClustersDistanceParameters | None],
    "opt_start_startval": typing.NotRequired[int | None],
})
CiftiFindClustersParametersTagged = typing.TypedDict('CiftiFindClustersParametersTagged', {
    "@type": typing.Literal["workbench/cifti-find-clusters"],
    "cifti": InputPathType,
    "surface_value_threshold": float,
    "surface_minimum_area": float,
    "volume_value_threshold": float,
    "volume_minimum_size": float,
    "direction": str,
    "cifti_out": str,
    "opt_less_than": bool,
    "left_surface": typing.NotRequired[CiftiFindClustersLeftSurfaceParameters | None],
    "right_surface": typing.NotRequired[CiftiFindClustersRightSurfaceParameters | None],
    "cerebellum_surface": typing.NotRequired[CiftiFindClustersCerebellumSurfaceParameters | None],
    "opt_cifti_roi_roi_cifti": typing.NotRequired[InputPathType | None],
    "opt_merged_volume": bool,
    "size_ratio": typing.NotRequired[CiftiFindClustersSizeRatioParameters | None],
    "distance": typing.NotRequired[CiftiFindClustersDistanceParameters | None],
    "opt_start_startval": typing.NotRequired[int | None],
})


def cifti_find_clusters_left_surface_params(
    surface: InputPathType,
    opt_corrected_areas_area_metric: InputPathType | None = None,
) -> CiftiFindClustersLeftSurfaceParametersTagged:
    """
    Build parameters.
    
    Args:
        surface: the left surface file.
        opt_corrected_areas_area_metric: vertex areas to use instead of\
            computing them from the surface: the corrected vertex areas, as a\
            metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "left_surface",
        "surface": surface,
    }
    if opt_corrected_areas_area_metric is not None:
        params["opt_corrected_areas_area_metric"] = opt_corrected_areas_area_metric
    return params


def cifti_find_clusters_left_surface_cargs(
    params: CiftiFindClustersLeftSurfaceParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-left-surface")
    cargs.append(execution.input_file(params.get("surface", None)))
    if params.get("opt_corrected_areas_area_metric", None) is not None:
        cargs.extend([
            "-corrected-areas",
            execution.input_file(params.get("opt_corrected_areas_area_metric", None))
        ])
    return cargs


def cifti_find_clusters_right_surface_params(
    surface: InputPathType,
    opt_corrected_areas_area_metric: InputPathType | None = None,
) -> CiftiFindClustersRightSurfaceParametersTagged:
    """
    Build parameters.
    
    Args:
        surface: the right surface file.
        opt_corrected_areas_area_metric: vertex areas to use instead of\
            computing them from the surface: the corrected vertex areas, as a\
            metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "right_surface",
        "surface": surface,
    }
    if opt_corrected_areas_area_metric is not None:
        params["opt_corrected_areas_area_metric"] = opt_corrected_areas_area_metric
    return params


def cifti_find_clusters_right_surface_cargs(
    params: CiftiFindClustersRightSurfaceParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-right-surface")
    cargs.append(execution.input_file(params.get("surface", None)))
    if params.get("opt_corrected_areas_area_metric", None) is not None:
        cargs.extend([
            "-corrected-areas",
            execution.input_file(params.get("opt_corrected_areas_area_metric", None))
        ])
    return cargs


def cifti_find_clusters_cerebellum_surface_params(
    surface: InputPathType,
    opt_corrected_areas_area_metric: InputPathType | None = None,
) -> CiftiFindClustersCerebellumSurfaceParametersTagged:
    """
    Build parameters.
    
    Args:
        surface: the cerebellum surface file.
        opt_corrected_areas_area_metric: vertex areas to use instead of\
            computing them from the surface: the corrected vertex areas, as a\
            metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cerebellum_surface",
        "surface": surface,
    }
    if opt_corrected_areas_area_metric is not None:
        params["opt_corrected_areas_area_metric"] = opt_corrected_areas_area_metric
    return params


def cifti_find_clusters_cerebellum_surface_cargs(
    params: CiftiFindClustersCerebellumSurfaceParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-cerebellum-surface")
    cargs.append(execution.input_file(params.get("surface", None)))
    if params.get("opt_corrected_areas_area_metric", None) is not None:
        cargs.extend([
            "-corrected-areas",
            execution.input_file(params.get("opt_corrected_areas_area_metric", None))
        ])
    return cargs


def cifti_find_clusters_size_ratio_params(
    surface_ratio: float,
    volume_ratio: float,
) -> CiftiFindClustersSizeRatioParametersTagged:
    """
    Build parameters.
    
    Args:
        surface_ratio: fraction of the structure's largest cluster area.
        volume_ratio: fraction of the structure's largest cluster volume.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "size_ratio",
        "surface_ratio": surface_ratio,
        "volume_ratio": volume_ratio,
    }
    return params


def cifti_find_clusters_size_ratio_cargs(
    params: CiftiFindClustersSizeRatioParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-size-ratio")
    cargs.append(str(params.get("surface_ratio", None)))
    cargs.append(str(params.get("volume_ratio", None)))
    return cargs


def cifti_find_clusters_distance_params(
    surface_distance: float,
    volume_distance: float,
) -> CiftiFindClustersDistanceParametersTagged:
    """
    Build parameters.
    
    Args:
        surface_distance: how far from the largest cluster a cluster can be,\
            edge to edge, in mm.
        volume_distance: how far from the largest cluster a cluster can be,\
            edge to edge, in mm.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "distance",
        "surface_distance": surface_distance,
        "volume_distance": volume_distance,
    }
    return params


def cifti_find_clusters_distance_cargs(
    params: CiftiFindClustersDistanceParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-distance")
    cargs.append(str(params.get("surface_distance", None)))
    cargs.append(str(params.get("volume_distance", None)))
    return cargs


class CiftiFindClustersOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiFindClustersParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti"""


def cifti_find_clusters_params(
    cifti: InputPathType,
    surface_value_threshold: float,
    surface_minimum_area: float,
    volume_value_threshold: float,
    volume_minimum_size: float,
    direction: str,
    cifti_out: str,
    opt_less_than: bool = False,
    left_surface: CiftiFindClustersLeftSurfaceParameters | None = None,
    right_surface: CiftiFindClustersRightSurfaceParameters | None = None,
    cerebellum_surface: CiftiFindClustersCerebellumSurfaceParameters | None = None,
    opt_cifti_roi_roi_cifti: InputPathType | None = None,
    opt_merged_volume: bool = False,
    size_ratio: CiftiFindClustersSizeRatioParameters | None = None,
    distance: CiftiFindClustersDistanceParameters | None = None,
    opt_start_startval: int | None = None,
) -> CiftiFindClustersParametersTagged:
    """
    Build parameters.
    
    Args:
        cifti: the input cifti.
        surface_value_threshold: threshold for surface data values.
        surface_minimum_area: threshold for surface cluster area, in mm^2.
        volume_value_threshold: threshold for volume data values.
        volume_minimum_size: threshold for volume cluster size, in mm^3.
        direction: which dimension to use for spatial information, ROW or\
            COLUMN.
        cifti_out: the output cifti.
        opt_less_than: find values less than <value-threshold>, rather than\
            greater.
        left_surface: specify the left surface to use.
        right_surface: specify the right surface to use.
        cerebellum_surface: specify the cerebellum surface to use.
        opt_cifti_roi_roi_cifti: search only within regions of interest: the\
            regions to search within, as a cifti file.
        opt_merged_volume: treat volume components as if they were a single\
            component.
        size_ratio: ignore clusters smaller than a given fraction of the\
            largest cluster in the structure.
        distance: ignore clusters further than a given distance from the\
            largest cluster in the structure.
        opt_start_startval: start labeling clusters from a value other than 1:\
            the value to give the first cluster found.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-find-clusters",
        "cifti": cifti,
        "surface_value_threshold": surface_value_threshold,
        "surface_minimum_area": surface_minimum_area,
        "volume_value_threshold": volume_value_threshold,
        "volume_minimum_size": volume_minimum_size,
        "direction": direction,
        "cifti_out": cifti_out,
        "opt_less_than": opt_less_than,
        "opt_merged_volume": opt_merged_volume,
    }
    if left_surface is not None:
        params["left_surface"] = left_surface
    if right_surface is not None:
        params["right_surface"] = right_surface
    if cerebellum_surface is not None:
        params["cerebellum_surface"] = cerebellum_surface
    if opt_cifti_roi_roi_cifti is not None:
        params["opt_cifti_roi_roi_cifti"] = opt_cifti_roi_roi_cifti
    if size_ratio is not None:
        params["size_ratio"] = size_ratio
    if distance is not None:
        params["distance"] = distance
    if opt_start_startval is not None:
        params["opt_start_startval"] = opt_start_startval
    return params


def cifti_find_clusters_cargs(
    params: CiftiFindClustersParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-cifti-find-clusters")
    cargs.append(execution.input_file(params.get("cifti", None)))
    cargs.append(str(params.get("surface_value_threshold", None)))
    cargs.append(str(params.get("surface_minimum_area", None)))
    cargs.append(str(params.get("volume_value_threshold", None)))
    cargs.append(str(params.get("volume_minimum_size", None)))
    cargs.append(params.get("direction", None))
    cargs.append(params.get("cifti_out", None))
    if params.get("opt_less_than", False):
        cargs.append("-less-than")
    if params.get("left_surface", None) is not None:
        cargs.extend(cifti_find_clusters_left_surface_cargs(params.get("left_surface", None), execution))
    if params.get("right_surface", None) is not None:
        cargs.extend(cifti_find_clusters_right_surface_cargs(params.get("right_surface", None), execution))
    if params.get("cerebellum_surface", None) is not None:
        cargs.extend(cifti_find_clusters_cerebellum_surface_cargs(params.get("cerebellum_surface", None), execution))
    if params.get("opt_cifti_roi_roi_cifti", None) is not None:
        cargs.extend([
            "-cifti-roi",
            execution.input_file(params.get("opt_cifti_roi_roi_cifti", None))
        ])
    if params.get("opt_merged_volume", False):
        cargs.append("-merged-volume")
    if params.get("size_ratio", None) is not None:
        cargs.extend(cifti_find_clusters_size_ratio_cargs(params.get("size_ratio", None), execution))
    if params.get("distance", None) is not None:
        cargs.extend(cifti_find_clusters_distance_cargs(params.get("distance", None), execution))
    if params.get("opt_start_startval", None) is not None:
        cargs.extend([
            "-start",
            str(params.get("opt_start_startval", None))
        ])
    return cargs


def cifti_find_clusters_outputs(
    params: CiftiFindClustersParameters,
    execution: Execution,
) -> CiftiFindClustersOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiFindClustersOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti_out", None)),
    )
    return ret


def cifti_find_clusters_execute(
    params: CiftiFindClustersParameters,
    runner: Runner | None = None,
) -> CiftiFindClustersOutputs:
    """
    cifti-find-clusters
    
    Filter clusters by area/volume.
    
    Outputs a cifti file with nonzero integers for all brainordinates within a
    large enough cluster, and zeros elsewhere. The integers denote cluster
    membership (by default, first cluster found will use value 1, second cluster
    2, etc). Cluster values are not reused across maps of the output, but
    instead keep counting up. The input cifti file must have a brain models
    mapping on the chosen dimension, columns for .dtseries, and either for
    .dconn. The ROI should have a brain models mapping along columns, exactly
    matching the mapping of the chosen direction in the input file. Data outside
    the ROI is ignored.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiFindClustersOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_FIND_CLUSTERS_METADATA)
    params = execution.params(params)
    cargs = cifti_find_clusters_cargs(params, execution)
    ret = cifti_find_clusters_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_find_clusters(
    cifti: InputPathType,
    surface_value_threshold: float,
    surface_minimum_area: float,
    volume_value_threshold: float,
    volume_minimum_size: float,
    direction: str,
    cifti_out: str,
    opt_less_than: bool = False,
    left_surface: CiftiFindClustersLeftSurfaceParameters | None = None,
    right_surface: CiftiFindClustersRightSurfaceParameters | None = None,
    cerebellum_surface: CiftiFindClustersCerebellumSurfaceParameters | None = None,
    opt_cifti_roi_roi_cifti: InputPathType | None = None,
    opt_merged_volume: bool = False,
    size_ratio: CiftiFindClustersSizeRatioParameters | None = None,
    distance: CiftiFindClustersDistanceParameters | None = None,
    opt_start_startval: int | None = None,
    runner: Runner | None = None,
) -> CiftiFindClustersOutputs:
    """
    cifti-find-clusters
    
    Filter clusters by area/volume.
    
    Outputs a cifti file with nonzero integers for all brainordinates within a
    large enough cluster, and zeros elsewhere. The integers denote cluster
    membership (by default, first cluster found will use value 1, second cluster
    2, etc). Cluster values are not reused across maps of the output, but
    instead keep counting up. The input cifti file must have a brain models
    mapping on the chosen dimension, columns for .dtseries, and either for
    .dconn. The ROI should have a brain models mapping along columns, exactly
    matching the mapping of the chosen direction in the input file. Data outside
    the ROI is ignored.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        cifti: the input cifti.
        surface_value_threshold: threshold for surface data values.
        surface_minimum_area: threshold for surface cluster area, in mm^2.
        volume_value_threshold: threshold for volume data values.
        volume_minimum_size: threshold for volume cluster size, in mm^3.
        direction: which dimension to use for spatial information, ROW or\
            COLUMN.
        cifti_out: the output cifti.
        opt_less_than: find values less than <value-threshold>, rather than\
            greater.
        left_surface: specify the left surface to use.
        right_surface: specify the right surface to use.
        cerebellum_surface: specify the cerebellum surface to use.
        opt_cifti_roi_roi_cifti: search only within regions of interest: the\
            regions to search within, as a cifti file.
        opt_merged_volume: treat volume components as if they were a single\
            component.
        size_ratio: ignore clusters smaller than a given fraction of the\
            largest cluster in the structure.
        distance: ignore clusters further than a given distance from the\
            largest cluster in the structure.
        opt_start_startval: start labeling clusters from a value other than 1:\
            the value to give the first cluster found.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiFindClustersOutputs`).
    """
    params = cifti_find_clusters_params(
        cifti=cifti,
        surface_value_threshold=surface_value_threshold,
        surface_minimum_area=surface_minimum_area,
        volume_value_threshold=volume_value_threshold,
        volume_minimum_size=volume_minimum_size,
        direction=direction,
        cifti_out=cifti_out,
        opt_less_than=opt_less_than,
        left_surface=left_surface,
        right_surface=right_surface,
        cerebellum_surface=cerebellum_surface,
        opt_cifti_roi_roi_cifti=opt_cifti_roi_roi_cifti,
        opt_merged_volume=opt_merged_volume,
        size_ratio=size_ratio,
        distance=distance,
        opt_start_startval=opt_start_startval,
    )
    return cifti_find_clusters_execute(params, runner)


__all__ = [
    "CIFTI_FIND_CLUSTERS_METADATA",
    "CiftiFindClustersOutputs",
    "cifti_find_clusters",
    "cifti_find_clusters_cerebellum_surface_params",
    "cifti_find_clusters_distance_params",
    "cifti_find_clusters_execute",
    "cifti_find_clusters_left_surface_params",
    "cifti_find_clusters_params",
    "cifti_find_clusters_right_surface_params",
    "cifti_find_clusters_size_ratio_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_GRADIENT_METADATA = Metadata(
    id="73cff8885780496a9b94b061ae3494c8870f981c.boutiques",
    name="volume-gradient",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


VolumeGradientPresmoothParameters = typing.TypedDict('VolumeGradientPresmoothParameters', {
    "@type": typing.NotRequired[typing.Literal["presmooth"]],
    "kernel": float,
    "opt_fwhm": bool,
})
VolumeGradientPresmoothParametersTagged = typing.TypedDict('VolumeGradientPresmoothParametersTagged', {
    "@type": typing.Literal["presmooth"],
    "kernel": float,
    "opt_fwhm": bool,
})


VolumeGradientParameters = typing.TypedDict('VolumeGradientParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/volume-gradient"]],
    "volume_in": InputPathType,
    "volume_out": str,
    "presmooth": typing.NotRequired[VolumeGradientPresmoothParameters | None],
    "opt_roi_roi_volume": typing.NotRequired[InputPathType | None],
    "opt_vectors_vector_volume_out": typing.NotRequired[str | None],
    "opt_subvolume_subvol": typing.NotRequired[str | None],
})
VolumeGradientParametersTagged = typing.TypedDict('VolumeGradientParametersTagged', {
    "@type": typing.Literal["workbench/volume-gradient"],
    "volume_in": InputPathType,
    "volume_out": str,
    "presmooth": typing.NotRequired[VolumeGradientPresmoothParameters | None],
    "opt_roi_roi_volume": typing.NotRequired[InputPathType | None],
    "opt_vectors_vector_volume_out": typing.NotRequired[str | None],
    "opt_subvolume_subvol": typing.NotRequired[str | None],
})


def volume_gradient_presmooth_params(
    kernel: float,
    opt_fwhm: bool = False,
) -> VolumeGradientPresmoothParametersTagged:
    """
    Build parameters.
    
    Args:
        kernel: the size of the gaussian smoothing kernel in mm, as sigma by\
            default.
        opt_fwhm: kernel size is FWHM, not sigma.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "presmooth",
        "kernel": kernel,
        "opt_fwhm": opt_fwhm,
    }
    return params


def volume_gradient_presmooth_cargs(
    params: VolumeGradientPresmoothParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-presmooth")
    cargs.append(str(params.get("kernel", None)))
    if params.get("opt_fwhm", False):
        cargs.append("-fwhm")
    return cargs


class VolumeGradientOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeGradientParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output gradient magnitude volume"""
    opt_vectors_vector_volume_out: OutputPathType | None
    """output vectors: the vectors as a volume file"""


def volume_gradient_params(
    volume_in: InputPathType,
    volume_out: str,
    presmooth: VolumeGradientPresmoothParameters | None = None,
    opt_roi_roi_volume: InputPathType | None = None,
    opt_vectors_vector_volume_out: str | None = None,
    opt_subvolume_subvol: str | None = None,
) -> VolumeGradientParametersTagged:
    """
    Build parameters.
    
    Args:
        volume_in: the input volume.
        volume_out: the output gradient magnitude volume.
        presmooth: smooth the volume before computing the gradient.
        opt_roi_roi_volume: select a region of interest to take the gradient\
            of: the region to take the gradient within.
        opt_vectors_vector_volume_out: output vectors: the vectors as a volume\
            file.
        opt_subvolume_subvol: select a single subvolume to take the gradient\
            of: the subvolume number or name.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-gradient",
        "volume_in": volume_in,
        "volume_out": volume_out,
    }
    if presmooth is not None:
        params["presmooth"] = presmooth
    if opt_roi_roi_volume is not None:
        params["opt_roi_roi_volume"] = opt_roi_roi_volume
    if opt_vectors_vector_volume_out is not None:
        params["opt_vectors_vector_volume_out"] = opt_vectors_vector_volume_out
    if opt_subvolume_subvol is not None:
        params["opt_subvolume_subvol"] = opt_subvolume_subvol
    return params


def volume_gradient_cargs(
    params: VolumeGradientParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-volume-gradient")
    cargs.append(execution.input_file(params.get("volume_in", None)))
    cargs.append(params.get("volume_out", None))
    if params.get("presmooth", None) is not None:
        cargs.extend(volume_gradient_presmooth_cargs(params.get("presmooth", None), execution))
    if params.get("opt_roi_roi_volume", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("opt_roi_roi_volume", None))
        ])
    if params.get("opt_vectors_vector_volume_out", None) is not None:
        cargs.extend([
            "-vectors",
            params.get("opt_vectors_vector_volume_out", None)
        ])
    if params.get("opt_subvolume_subvol", None) is not None:
        cargs.extend([
            "-subvolume",
            params.get("opt_subvolume_subvol", None)
        ])
    return cargs


def volume_gradient_outputs(
    params: VolumeGradientParameters,
    execution: Execution,
) -> VolumeGradientOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeGradientOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume_out", None)),
        opt_vectors_vector_volume_out=execution.output_file(params.get("opt_vectors_vector_volume_out", None)) if (params.get("opt_vectors_vector_volume_out") is not None) else None,
    )
    return ret


def volume_gradient_execute(
    params: VolumeGradientParameters,
    runner: Runner | None = None,
) -> VolumeGradientOutputs:
    """
    volume-gradient
    
    Gradient of a volume file.
    
    Computes the gradient of the volume by doing linear regressions for each
    voxel, considering only its face neighbors unless too few face neighbors
    exist. The gradient vector is constructed from the partial derivatives of
    the resulting linear function, and the magnitude of this vector is the
    output. If specified, the volume vector output is arranged with the x, y,
    and z components from a subvolume as consecutive subvolumes.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeGradientOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_GRADIENT_METADATA)
    params = execution.params(params)
    cargs = volume_gradient_cargs(params, execution)
    ret = volume_gradient_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_gradient(
    volume_in: InputPathType,
    volume_out: str,
    presmooth: VolumeGradientPresmoothParameters | None = None,
    opt_roi_roi_volume: InputPathType | None = None,
    opt_vectors_vector_volume_out: str | None = None,
    opt_subvolume_subvol: str | None = None,
    runner: Runner | None = None,
) -> VolumeGradientOutputs:
    """
    volume-gradient
    
    Gradient of a volume file.
    
    Computes the gradient of the volume by doing linear regressions for each
    voxel, considering only its face neighbors unless too few face neighbors
    exist. The gradient vector is constructed from the partial derivatives of
    the resulting linear function, and the magnitude of this vector is the
    output. If specified, the volume vector output is arranged with the x, y,
    and z components from a subvolume as consecutive subvolumes.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        volume_in: the input volume.
        volume_out: the output gradient magnitude volume.
        presmooth: smooth the volume before computing the gradient.
        opt_roi_roi_volume: select a region of interest to take the gradient\
            of: the region to take the gradient within.
        opt_vectors_vector_volume_out: output vectors: the vectors as a volume\
            file.
        opt_subvolume_subvol: select a single subvolume to take the gradient\
            of: the subvolume number or name.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeGradientOutputs`).
    """
    params = volume_gradient_params(
        volume_in=volume_in,
        volume_out=volume_out,
        presmooth=presmooth,
        opt_roi_roi_volume=opt_roi_roi_volume,
        opt_vectors_vector_volume_out=opt_vectors_vector_volume_out,
        opt_subvolume_subvol=opt_subvolume_subvol,
    )
    return volume_gradient_execute(params, runner)


__all__ = [
    "VOLUME_GRADIENT_METADATA",
    "VolumeGradientOutputs",
    "volume_gradient",
    "volume_gradient_execute",
    "volume_gradient_params",
    "volume_gradient_presmooth_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_TO_VOLUME_MAPPING_METADATA = Metadata(
    id="4a17edeb9af7773ec08dedde028a48523ef336b2.boutiques",
    name="metric-to-volume-mapping",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


MetricToVolumeMappingRibbonConstrainedParameters = typing.TypedDict('MetricToVolumeMappingRibbonConstrainedParameters', {
    "@type": typing.NotRequired[typing.Literal["ribbon_constrained"]],
    "inner_surf": InputPathType,
    "outer_surf": InputPathType,
    "opt_voxel_subdiv_subdiv_num": typing.NotRequired[int | None],
    "opt_greedy": bool,
    "opt_thick_columns": bool,
})
MetricToVolumeMappingRibbonConstrainedParametersTagged = typing.TypedDict('MetricToVolumeMappingRibbonConstrainedParametersTagged', {
    "@type": typing.Literal["ribbon_constrained"],
    "inner_surf": InputPathType,
    "outer_surf": InputPathType,
    "opt_voxel_subdiv_subdiv_num": typing.NotRequired[int | None],
    "opt_greedy": bool,
    "opt_thick_columns": bool,
})


MetricToVolumeMappingParameters = typing.TypedDict('MetricToVolumeMappingParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/metric-to-volume-mapping"]],
    "metric": InputPathType,
    "surface": InputPathType,
    "volume_space": InputPathType,
    "volume_out": str,
    "opt_nearest_vertex_distance": typing.NotRequired[float | None],
    "ribbon_constrained": typing.NotRequired[MetricToVolumeMappingRibbonConstrainedParameters | None],
})
MetricToVolumeMappingParametersTagged = typing.TypedDict('MetricToVolumeMappingParametersTagged', {
    "@type": typing.Literal["workbench/metric-to-volume-mapping"],
    "metric": InputPathType,
    "surface": InputPathType,
    "volume_space": InputPathType,
    "volume_out": str,
    "opt_nearest_vertex_distance": typing.NotRequired[float | None],
    "ribbon_constrained": typing.NotRequired[MetricToVolumeMappingRibbonConstrainedParameters | None],
})


def metric_to_volume_mapping_ribbon_constrained_params(
    inner_surf: InputPathType,
    outer_surf: InputPathType,
    opt_voxel_subdiv_subdiv_num: int | None = None,
    opt_greedy: bool = False,
    opt_thick_columns: bool = False,
) -> MetricToVolumeMappingRibbonConstrainedParametersTagged:
    """
    Build parameters.
    
    Args:
        inner_surf: the inner surface of the ribbon.
        outer_surf: the outer surface of the ribbon.
        opt_voxel_subdiv_subdiv_num: voxel divisions while estimating voxel\
            weights: number of subdivisions, default 3.
        opt_greedy: instead of antialiasing partial-volumed voxels, put full\
            metric values (legacy behavior).
        opt_thick_columns: use overlapping columns (legacy method).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ribbon_constrained",
        "inner_surf": inner_surf,
        "outer_surf": outer_surf,
        "opt_greedy": opt_greedy,
        "opt_thick_columns": opt_thick_columns,
    }
    if opt_voxel_subdiv_subdiv_num is not None:
        params["opt_voxel_subdiv_subdiv_num"] = opt_voxel_subdiv_subdiv_num
    return params


def metric_to_volume_mapping_ribbon_constrained_cargs(
    params: MetricToVolumeMappingRibbonConstrainedParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-ribbon-constrained")
    cargs.append(execution.input_file(params.get("inner_surf", None)))
    cargs.append(execution.input_file(params.get("outer_surf", None)))
    if params.get("opt_voxel_subdiv_subdiv_num", None) is not None:
        cargs.extend([
            "-voxel-subdiv",
            str(params.get("opt_voxel_subdiv_subdiv_num", None))
        ])
    if params.get("opt_greedy", False):
        cargs.append("-greedy")
    if params.get("opt_thick_columns", False):
        cargs.append("-thick-columns")
    return cargs


class MetricToVolumeMappingOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricToVolumeMappingParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume file"""


def metric_to_volume_mapping_params(
    metric: InputPathType,
    surface: InputPathType,
    volume_space: InputPathType,
    volume_out: str,
    opt_nearest_vertex_distance: float | None = None,
    ribbon_constrained: MetricToVolumeMappingRibbonConstrainedParameters | None = None,
) -> MetricToVolumeMappingParametersTagged:
    """
    Build parameters.
    
    Args:
        metric: the input metric file.
        surface: the surface to use coordinates from.
        volume_space: a volume file in the desired output volume space.
        volume_out: the output volume file.
        opt_nearest_vertex_distance: use the value from the vertex closest to\
            the voxel center: how far from the surface to map values to voxels, in\
            mm.
        ribbon_constrained: use ribbon constrained mapping algorithm.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-to-volume-mapping",
        "metric": metric,
        "surface": surface,
        "volume_space": volume_space,
        "volume_out": volume_out,
    }
    if opt_nearest_vertex_distance is not None:
        params["opt_nearest_vertex_distance"] = opt_nearest_vertex_distance
    if ribbon_constrained is not None:
        params["ribbon_constrained"] = ribbon_constrained
    return params


def metric_to_volume_mapping_cargs(
    params: MetricToVolumeMappingParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-metric-to-volume-mapping")
    cargs.append(execution.input_file(params.get("metric", None)))
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(execution.input_file(params.get("volume_space", None)))
    cargs.append(params.get("volume_out", None))
    if params.get("opt_nearest_vertex_distance", None) is not None:
        cargs.extend([
            "-nearest-vertex",
            str(params.get("opt_nearest_vertex_distance", None))
        ])
    if params.get("ribbon_constrained", None) is not None:
        cargs.extend(metric_to_volume_mapping_ribbon_constrained_cargs(params.get("ribbon_constrained", None), execution))
    return cargs


def metric_to_volume_mapping_outputs(
    params: MetricToVolumeMappingParameters,
    execution: Execution,
) -> MetricToVolumeMappingOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricToVolumeMappingOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume_out", None)),
    )
    return ret


def metric_to_volume_mapping_execute(
    params: MetricToVolumeMappingParameters,
    runner: Runner | None = None,
) -> MetricToVolumeMappingOutputs:
    """
    metric-to-volume-mapping
    
    Map metric file to volume.
    
    Maps values from a metric file into a volume file. You must specify exactly
    one mapping method option. The -nearest-vertex method uses the value from
    the vertex closest to the voxel center (useful for integer values). The
    -ribbon-constrained method uses the same method as in
    -volume-to-surface-mapping, then uses the weights in reverse. Mapping to
    lower resolutions than the mesh may require a larger -voxel-subdiv value in
    order to have all of the surface data participate.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricToVolumeMappingOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_TO_VOLUME_MAPPING_METADATA)
    params = execution.params(params)
    cargs = metric_to_volume_mapping_cargs(params, execution)
    ret = metric_to_volume_mapping_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_to_volume_mapping(
    metric: InputPathType,
    surface: InputPathType,
    volume_space: InputPathType,
    volume_out: str,
    opt_nearest_vertex_distance: float | None = None,
    ribbon_constrained: MetricToVolumeMappingRibbonConstrainedParameters | None = None,
    runner: Runner | None = None,
) -> MetricToVolumeMappingOutputs:
    """
    metric-to-volume-mapping
    
    Map metric file to volume.
    
    Maps values from a metric file into a volume file. You must specify exactly
    one mapping method option. The -nearest-vertex method uses the value from
    the vertex closest to the voxel center (useful for integer values). The
    -ribbon-constrained method uses the same method as in
    -volume-to-surface-mapping, then uses the weights in reverse. Mapping to
    lower resolutions than the mesh may require a larger -voxel-subdiv value in
    order to have all of the surface data participate.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        metric: the input metric file.
        surface: the surface to use coordinates from.
        volume_space: a volume file in the desired output volume space.
        volume_out: the output volume file.
        opt_nearest_vertex_distance: use the value from the vertex closest to\
            the voxel center: how far from the surface to map values to voxels, in\
            mm.
        ribbon_constrained: use ribbon constrained mapping algorithm.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricToVolumeMappingOutputs`).
    """
    params = metric_to_volume_mapping_params(
        metric=metric,
        surface=surface,
        volume_space=volume_space,
        volume_out=volume_out,
        opt_nearest_vertex_distance=opt_nearest_vertex_distance,
        ribbon_constrained=ribbon_constrained,
    )
    return metric_to_volume_mapping_execute(params, runner)


__all__ = [
    "METRIC_TO_VOLUME_MAPPING_METADATA",
    "MetricToVolumeMappingOutputs",
    "metric_to_volume_mapping",
    "metric_to_volume_mapping_execute",
    "metric_to_volume_mapping_params",
    "metric_to_volume_mapping_ribbon_constrained_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_PARCELLATE_METADATA = Metadata(
    id="09eebc79465dbc1545219d2adbd3332cbcb71232.boutiques",
    name="cifti-parcellate",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


CiftiParcellateSpatialWeightsParameters = typing.TypedDict('CiftiParcellateSpatialWeightsParameters', {
    "@type": typing.NotRequired[typing.Literal["spatial_weights"]],
    "opt_left_area_surf_left_surf": typing.NotRequired[InputPathType | None],
    "opt_right_area_surf_right_surf": typing.NotRequired[InputPathType | None],
    "opt_cerebellum_area_surf_cerebellum_surf": typing.NotRequired[InputPathType | None],
    "opt_left_area_metric_left_metric": typing.NotRequired[InputPathType | None],
    "opt_right_area_metric_right_metric": typing.NotRequired[InputPathType | None],
    "opt_cerebellum_area_metric_cerebellum_metric": typing.NotRequired[InputPathType | None],
})
CiftiParcellateSpatialWeightsParametersTagged = typing.TypedDict('CiftiParcellateSpatialWeightsParametersTagged', {
    "@type": typing.Literal["spatial_weights"],
    "opt_left_area_surf_left_surf": typing.NotRequired[InputPathType | None],
    "opt_right_area_surf_right_surf": typing.NotRequired[InputPathType | None],
    "opt_cerebellum_area_surf_cerebellum_surf": typing.NotRequired[InputPathType | None],
    "opt_left_area_metric_left_metric": typing.NotRequired[InputPathType | None],
    "opt_right_area_metric_right_metric": typing.NotRequired[InputPathType | None],
    "opt_cerebellum_area_metric_cerebellum_metric": typing.NotRequired[InputPathType | None],
})


CiftiParcellateExcludeOutliersParameters = typing.TypedDict('CiftiParcellateExcludeOutliersParameters', {
    "@type": typing.NotRequired[typing.Literal["exclude_outliers"]],
    "sigma_below": float,
    "sigma_above": float,
})
CiftiParcellateExcludeOutliersParametersTagged = typing.TypedDict('CiftiParcellateExcludeOutliersParametersTagged', {
    "@type": typing.Literal["exclude_outliers"],
    "sigma_below": float,
    "sigma_above": float,
})


CiftiParcellateParameters = typing.TypedDict('CiftiParcellateParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/cifti-parcellate"]],
    "cifti_in": InputPathType,
    "cifti_label": InputPathType,
    "direction": str,
    "cifti_out": str,
    "spatial_weights": typing.NotRequired[CiftiParcellateSpatialWeightsParameters | None],
    "opt_cifti_weights_weight_cifti": typing.NotRequired[InputPathType | None],
    "opt_method_method": typing.NotRequired[str | None],
    "exclude_outliers": typing.NotRequired[CiftiParcellateExcludeOutliersParameters | None],
    "opt_only_numeric": bool,
    "opt_fill_value_value": typing.NotRequired[float | None],
    "opt_nonempty_mask_out_mask_out": typing.NotRequired[str | None],
    "opt_legacy_mode": bool,
    "opt_include_empty": bool,
})
CiftiParcellateParametersTagged = typing.TypedDict('CiftiParcellateParametersTagged', {
    "@type": typing.Literal["workbench/cifti-parcellate"],
    "cifti_in": InputPathType,
    "cifti_label": InputPathType,
    "direction": str,
    "cifti_out": str,
    "spatial_weights": typing.NotRequired[CiftiParcellateSpatialWeightsParameters | None],
    "opt_cifti_weights_weight_cifti": typing.NotRequired[InputPathType | None],
    "opt_method_method": typing.NotRequired[str | None],
    "exclude_outliers": typing.NotRequired[CiftiParcellateExcludeOutliersParameters | None],
    "opt_only_numeric": bool,
    "opt_fill_value_value": typing.NotRequired[float | None],
    "opt_nonempty_mask_out_mask_out": typing.NotRequired[str | None],
    "opt_legacy_mode": bool,
    "opt_include_empty": bool,
})


def cifti_parcellate_spatial_weights_params(
    opt_left_area_surf_left_surf: InputPathType | None = None,
    opt_right_area_surf_right_surf: InputPathType | None = None,
    opt_cerebellum_area_surf_cerebellum_surf: InputPathType | None = None,
    opt_left_area_metric_left_metric: InputPathType | None = None,
    opt_right_area_metric_right_metric: InputPathType | None = None,
    opt_cerebellum_area_metric_cerebellum_metric: InputPathType | None = None,
) -> CiftiParcellateSpatialWeightsParametersTagged:
    """
    Build parameters.
    
    Args:
        opt_left_area_surf_left_surf: use a surface for left vertex areas: the\
            left surface to use, areas are in mm^2.
        opt_right_area_surf_right_surf: use a surface for right vertex areas:\
            the right surface to use, areas are in mm^2.
        opt_cerebellum_area_surf_cerebellum_surf: use a surface for cerebellum\
            vertex areas: the cerebellum surface to use, areas are in mm^2.
        opt_left_area_metric_left_metric: use a metric file for left vertex\
            weights: metric file containing left vertex weights.
        opt_right_area_metric_right_metric: use a metric file for right vertex\
            weights: metric file containing right vertex weights.
        opt_cerebellum_area_metric_cerebellum_metric: use a metric file for\
            cerebellum vertex weights: metric file containing cerebellum vertex\
            weights.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "spatial_weights",
    }
    if opt_left_area_surf_left_surf is not None:
        params["opt_left_area_surf_left_surf"] = opt_left_area_surf_left_surf
    if opt_right_area_surf_right_surf is not None:
        params["opt_right_area_surf_right_surf"] = opt_right_area_surf_right_surf
    if opt_cerebellum_area_surf_cerebellum_surf is not None:
        params["opt_cerebellum_area_surf_cerebellum_surf"] = opt_cerebellum_area_surf_cerebellum_surf
    if opt_left_area_metric_left_metric is not None:
        params["opt_left_area_metric_left_metric"] = opt_left_area_metric_left_metric
    if opt_right_area_metric_right_metric is not None:
        params["opt_right_area_metric_right_metric"] = opt_right_area_metric_right_metric
    if opt_cerebellum_area_metric_cerebellum_metric is not None:
        params["opt_cerebellum_area_metric_cerebellum_metric"] = opt_cerebellum_area_metric_cerebellum_metric
    return params


def cifti_parcellate_spatial_weights_cargs(
    params: CiftiParcellateSpatialWeightsParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-spatial-weights")
    if params.get("opt_left_area_surf_left_surf", None) is not None:
        cargs.extend([
            "-left-area-surf",
            execution.input_file(params.get("opt_left_area_surf_left_surf", None))
        ])
    if params.get("opt_right_area_surf_right_surf", None) is not None:
        cargs.extend([
            "-right-area-surf",
            execution.input_file(params.get("opt_right_area_surf_right_surf", None))
        ])
    if params.get("opt_cerebellum_area_surf_cerebellum_surf", None) is not None:
        cargs.extend([
            "-cerebellum-area-surf",
            execution.input_file(params.get("opt_cerebellum_area_surf_cerebellum_surf", None))
        ])
    if params.get("opt_left_area_metric_left_metric", None) is not None:
        cargs.extend([
            "-left-area-metric",
            execution.input_file(params.get("opt_left_area_metric_left_metric", None))
        ])
    if params.get("opt_right_area_metric_right_metric", None) is not None:
        cargs.extend([
            "-right-area-metric",
            execution.input_file(params.get("opt_right_area_metric_right_metric", None))
        ])
    if params.get("opt_cerebellum_area_metric_cerebellum_metric", None) is not None:
        cargs.extend([
            "-cerebellum-area-metric",
            execution.input_file(params.get("opt_cerebellum_area_metric_cerebellum_metric", None))
        ])
    return cargs


def cifti_parcellate_exclude_outliers_params(
    sigma_below: float,
    sigma_above: float,
) -> CiftiParcellateExcludeOutliersParametersTagged:
    """
    Build parameters.
    
    Args:
        sigma_below: number of standard deviations below the mean to include.
        sigma_above: number of standard deviations above the mean to include.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "exclude_outliers",
        "sigma_below": sigma_below,
        "sigma_above": sigma_above,
    }
    return params


def cifti_parcellate_exclude_outliers_cargs(
    params: CiftiParcellateExcludeOutliersParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-exclude-outliers")
    cargs.append(str(params.get("sigma_below", None)))
    cargs.append(str(params.get("sigma_above", None)))
    return cargs


class CiftiParcellateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiParcellateParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """output cifti file"""
    opt_nonempty_mask_out_mask_out: OutputPathType | None
    """output a matching pscalar file that has 0s in empty parcels, and 1s
    elsewhere: the output mask file"""


def cifti_parcellate_params(
    cifti_in: InputPathType,
    cifti_label: InputPathType,
    direction: str,
    cifti_out: str,
    spatial_weights: CiftiParcellateSpatialWeightsParameters | None = None,
    opt_cifti_weights_weight_cifti: InputPathType | None = None,
    opt_method_method: str | None = None,
    exclude_outliers: CiftiParcellateExcludeOutliersParameters | None = None,
    opt_only_numeric: bool = False,
    opt_fill_value_value: float | None = None,
    opt_nonempty_mask_out_mask_out: str | None = None,
    opt_legacy_mode: bool = False,
    opt_include_empty: bool = False,
) -> CiftiParcellateParametersTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the cifti file to parcellate.
        cifti_label: a cifti label file to use for the parcellation.
        direction: which mapping to parcellate (integer, ROW, or COLUMN).
        cifti_out: output cifti file.
        spatial_weights: use voxel volume and either vertex areas or metric\
            files as weights.
        opt_cifti_weights_weight_cifti: use a cifti file containing weights:\
            the weights to use, as a cifti file.
        opt_method_method: specify method of parcellation (default MEAN, or\
            MODE if label data): the method to use to assign parcel values from the\
            values of member brainordinates.
        exclude_outliers: exclude non-numeric values and outliers from each\
            parcel by standard deviation.
        opt_only_numeric: exclude non-numeric values.
        opt_fill_value_value: specify value to use in empty parcels (default\
            0): the value to fill empty parcels with.
        opt_nonempty_mask_out_mask_out: output a matching pscalar file that has\
            0s in empty parcels, and 1s elsewhere: the output mask file.
        opt_legacy_mode: use the old behavior, parcels are defined by the\
            intersection between labels and valid data, and empty parcels are\
            discarded.
        opt_include_empty: deprecated: now the default behavior.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-parcellate",
        "cifti_in": cifti_in,
        "cifti_label": cifti_label,
        "direction": direction,
        "cifti_out": cifti_out,
        "opt_only_numeric": opt_only_numeric,
        "opt_legacy_mode": opt_legacy_mode,
        "opt_include_empty": opt_include_empty,
    }
    if spatial_weights is not None:
        params["spatial_weights"] = spatial_weights
    if opt_cifti_weights_weight_cifti is not None:
        params["opt_cifti_weights_weight_cifti"] = opt_cifti_weights_weight_cifti
    if opt_method_method is not None:
        params["opt_method_method"] = opt_method_method
    if exclude_outliers is not None:
        params["exclude_outliers"] = exclude_outliers
    if opt_fill_value_value is not None:
        params["opt_fill_value_value"] = opt_fill_value_value
    if opt_nonempty_mask_out_mask_out is not None:
        params["opt_nonempty_mask_out_mask_out"] = opt_nonempty_mask_out_mask_out
    return params


def cifti_parcellate_cargs(
    params: CiftiParcellateParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-cifti-parcellate")
    cargs.append(execution.input_file(params.get("cifti_in", None)))
    cargs.append(execution.input_file(params.get("cifti_label", None)))
    cargs.append(params.get("direction", None))
    cargs.append(params.get("cifti_out", None))
    if params.get("spatial_weights", None) is not None:
        cargs.extend(cifti_parcellate_spatial_weights_cargs(params.get("spatial_weights", None), execution))
    if params.get("opt_cifti_weights_weight_cifti", None) is not None:
        cargs.extend([
            "-cifti-weights",
            execution.input_file(params.get("opt_cifti_weights_weight_cifti", None))
        ])
    if params.get("opt_method_method", None) is not None:
        cargs.extend([
            "-method",
            params.get("opt_method_method", None)
        ])
    if params.get("exclude_outliers", None) is not None:
        cargs.extend(cifti_parcellate_exclude_outliers_cargs(params.get("exclude_outliers", None), execution))
    if params.get("opt_only_numeric", False):
        cargs.append("-only-numeric")
    if params.get("opt_fill_value_value", None) is not None:
        cargs.extend([
            "-fill-value",
            str(params.get("opt_fill_value_value", None))
        ])
    if params.get("opt_nonempty_mask_out_mask_out", None) is not None:
        cargs.extend([
            "-nonempty-mask-out",
            params.get("opt_nonempty_mask_out_mask_out", None)
        ])
    if params.get("opt_legacy_mode", False):
        cargs.append("-legacy-mode")
    if params.get("opt_include_empty", False):
        cargs.append("-include-empty")
    return cargs


def cifti_parcellate_outputs(
    params: CiftiParcellateParameters,
    execution: Execution,
) -> CiftiParcellateOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiParcellateOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti_out", None)),
        opt_nonempty_mask_out_mask_out=execution.output_file(params.get("opt_nonempty_mask_out_mask_out", None)) if (params.get("opt_nonempty_mask_out_mask_out") is not None) else None,
    )
    return ret


def cifti_parcellate_execute(
    params: CiftiParcellateParameters,
    runner: Runner | None = None,
) -> CiftiParcellateOutputs:
    """
    cifti-parcellate
    
    Parcellate a cifti file.
    
    Each label (other than the unlabeled key) in the cifti label file will be
    treated as a parcel, and all rows or columns of data within the parcel are
    averaged together to form the parcel's output row or column. If -legacy-mode
    is specified, parcels will be defined as the overlap between a label and the
    data, with no errors for missing data vertices or voxels, and empty parcels
    discarded. The direction can be either an integer starting from 1, or the
    strings 'ROW' or 'COLUMN'. For dtseries or dscalar, use COLUMN. If you are
    parcellating a dconn in both directions, parcellating by ROW first will use
    much less memory.
    
    NOTE: the parcels in the output file are sorted by the numeric label keys,
    in ascending order.
    
    The parameter to the -method option must be one of the following:
    
    MAX: the maximum value
    MIN: the minimum value
    INDEXMAX: the 1-based index of the maximum value
    INDEXMIN: the 1-based index of the minimum value
    SUM: add all values
    PRODUCT: multiply all values
    MEAN: the mean of the data
    STDEV: the standard deviation (N denominator)
    SAMPSTDEV: the sample standard deviation (N-1 denominator)
    VARIANCE: the variance of the data
    TSNR: mean divided by sample standard deviation (N-1 denominator)
    COV: sample standard deviation (N-1 denominator) divided by mean
    L2NORM: square root of sum of squares
    MEDIAN: the median of the data
    MODE: the mode of the data
    COUNT_NONZERO: the number of nonzero elements in the data
    
    The -*-weights options are mutually exclusive and may only be used with MEAN
    (default), SUM, STDEV, SAMPSTDEV, VARIANCE, MEDIAN, or MODE (default for
    label data).
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiParcellateOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_PARCELLATE_METADATA)
    params = execution.params(params)
    cargs = cifti_parcellate_cargs(params, execution)
    ret = cifti_parcellate_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_parcellate(
    cifti_in: InputPathType,
    cifti_label: InputPathType,
    direction: str,
    cifti_out: str,
    spatial_weights: CiftiParcellateSpatialWeightsParameters | None = None,
    opt_cifti_weights_weight_cifti: InputPathType | None = None,
    opt_method_method: str | None = None,
    exclude_outliers: CiftiParcellateExcludeOutliersParameters | None = None,
    opt_only_numeric: bool = False,
    opt_fill_value_value: float | None = None,
    opt_nonempty_mask_out_mask_out: str | None = None,
    opt_legacy_mode: bool = False,
    opt_include_empty: bool = False,
    runner: Runner | None = None,
) -> CiftiParcellateOutputs:
    """
    cifti-parcellate
    
    Parcellate a cifti file.
    
    Each label (other than the unlabeled key) in the cifti label file will be
    treated as a parcel, and all rows or columns of data within the parcel are
    averaged together to form the parcel's output row or column. If -legacy-mode
    is specified, parcels will be defined as the overlap between a label and the
    data, with no errors for missing data vertices or voxels, and empty parcels
    discarded. The direction can be either an integer starting from 1, or the
    strings 'ROW' or 'COLUMN'. For dtseries or dscalar, use COLUMN. If you are
    parcellating a dconn in both directions, parcellating by ROW first will use
    much less memory.
    
    NOTE: the parcels in the output file are sorted by the numeric label keys,
    in ascending order.
    
    The parameter to the -method option must be one of the following:
    
    MAX: the maximum value
    MIN: the minimum value
    INDEXMAX: the 1-based index of the maximum value
    INDEXMIN: the 1-based index of the minimum value
    SUM: add all values
    PRODUCT: multiply all values
    MEAN: the mean of the data
    STDEV: the standard deviation (N denominator)
    SAMPSTDEV: the sample standard deviation (N-1 denominator)
    VARIANCE: the variance of the data
    TSNR: mean divided by sample standard deviation (N-1 denominator)
    COV: sample standard deviation (N-1 denominator) divided by mean
    L2NORM: square root of sum of squares
    MEDIAN: the median of the data
    MODE: the mode of the data
    COUNT_NONZERO: the number of nonzero elements in the data
    
    The -*-weights options are mutually exclusive and may only be used with MEAN
    (default), SUM, STDEV, SAMPSTDEV, VARIANCE, MEDIAN, or MODE (default for
    label data).
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        cifti_in: the cifti file to parcellate.
        cifti_label: a cifti label file to use for the parcellation.
        direction: which mapping to parcellate (integer, ROW, or COLUMN).
        cifti_out: output cifti file.
        spatial_weights: use voxel volume and either vertex areas or metric\
            files as weights.
        opt_cifti_weights_weight_cifti: use a cifti file containing weights:\
            the weights to use, as a cifti file.
        opt_method_method: specify method of parcellation (default MEAN, or\
            MODE if label data): the method to use to assign parcel values from the\
            values of member brainordinates.
        exclude_outliers: exclude non-numeric values and outliers from each\
            parcel by standard deviation.
        opt_only_numeric: exclude non-numeric values.
        opt_fill_value_value: specify value to use in empty parcels (default\
            0): the value to fill empty parcels with.
        opt_nonempty_mask_out_mask_out: output a matching pscalar file that has\
            0s in empty parcels, and 1s elsewhere: the output mask file.
        opt_legacy_mode: use the old behavior, parcels are defined by the\
            intersection between labels and valid data, and empty parcels are\
            discarded.
        opt_include_empty: deprecated: now the default behavior.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiParcellateOutputs`).
    """
    params = cifti_parcellate_params(
        cifti_in=cifti_in,
        cifti_label=cifti_label,
        direction=direction,
        cifti_out=cifti_out,
        spatial_weights=spatial_weights,
        opt_cifti_weights_weight_cifti=opt_cifti_weights_weight_cifti,
        opt_method_method=opt_method_method,
        exclude_outliers=exclude_outliers,
        opt_only_numeric=opt_only_numeric,
        opt_fill_value_value=opt_fill_value_value,
        opt_nonempty_mask_out_mask_out=opt_nonempty_mask_out_mask_out,
        opt_legacy_mode=opt_legacy_mode,
        opt_include_empty=opt_include_empty,
    )
    return cifti_parcellate_execute(params, runner)


__all__ = [
    "CIFTI_PARCELLATE_METADATA",
    "CiftiParcellateOutputs",
    "cifti_parcellate",
    "cifti_parcellate_exclude_outliers_params",
    "cifti_parcellate_execute",
    "cifti_parcellate_params",
    "cifti_parcellate_spatial_weights_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_PARCEL_RESAMPLING_METADATA = Metadata(
    id="5bbe50c5806e2adef7cc3daf350744e6e52e6e64.boutiques",
    name="volume-parcel-resampling",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


VolumeParcelResamplingParameters = typing.TypedDict('VolumeParcelResamplingParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/volume-parcel-resampling"]],
    "volume_in": InputPathType,
    "cur_parcels": InputPathType,
    "new_parcels": InputPathType,
    "kernel": float,
    "volume_out": str,
    "opt_fix_zeros": bool,
    "opt_fwhm": bool,
    "opt_subvolume_subvol": typing.NotRequired[str | None],
})
VolumeParcelResamplingParametersTagged = typing.TypedDict('VolumeParcelResamplingParametersTagged', {
    "@type": typing.Literal["workbench/volume-parcel-resampling"],
    "volume_in": InputPathType,
    "cur_parcels": InputPathType,
    "new_parcels": InputPathType,
    "kernel": float,
    "volume_out": str,
    "opt_fix_zeros": bool,
    "opt_fwhm": bool,
    "opt_subvolume_subvol": typing.NotRequired[str | None],
})


class VolumeParcelResamplingOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeParcelResamplingParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """output volume"""


def volume_parcel_resampling_params(
    volume_in: InputPathType,
    cur_parcels: InputPathType,
    new_parcels: InputPathType,
    kernel: float,
    volume_out: str,
    opt_fix_zeros: bool = False,
    opt_fwhm: bool = False,
    opt_subvolume_subvol: str | None = None,
) -> VolumeParcelResamplingParametersTagged:
    """
    Build parameters.
    
    Args:
        volume_in: the input data volume.
        cur_parcels: label volume of where the parcels currently are.
        new_parcels: label volume of where the parcels should be.
        kernel: gaussian kernel size in mm to smooth by during resampling, as\
            sigma by default.
        volume_out: output volume.
        opt_fix_zeros: treat zero values as not being data.
        opt_fwhm: smoothing kernel size is FWHM, not sigma.
        opt_subvolume_subvol: select a single subvolume as input: the subvolume\
            number or name.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-parcel-resampling",
        "volume_in": volume_in,
        "cur_parcels": cur_parcels,
        "new_parcels": new_parcels,
        "kernel": kernel,
        "volume_out": volume_out,
        "opt_fix_zeros": opt_fix_zeros,
        "opt_fwhm": opt_fwhm,
    }
    if opt_subvolume_subvol is not None:
        params["opt_subvolume_subvol"] = opt_subvolume_subvol
    return params


def volume_parcel_resampling_cargs(
    params: VolumeParcelResamplingParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-volume-parcel-resampling")
    cargs.append(execution.input_file(params.get("volume_in", None)))
    cargs.append(execution.input_file(params.get("cur_parcels", None)))
    cargs.append(execution.input_file(params.get("new_parcels", None)))
    cargs.append(str(params.get("kernel", None)))
    cargs.append(params.get("volume_out", None))
    if params.get("opt_fix_zeros", False):
        cargs.append("-fix-zeros")
    if params.get("opt_fwhm", False):
        cargs.append("-fwhm")
    if params.get("opt_subvolume_subvol", None) is not None:
        cargs.extend([
            "-subvolume",
            params.get("opt_subvolume_subvol", None)
        ])
    return cargs


def volume_parcel_resampling_outputs(
    params: VolumeParcelResamplingParameters,
    execution: Execution,
) -> VolumeParcelResamplingOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeParcelResamplingOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume_out", None)),
    )
    return ret


def volume_parcel_resampling_execute(
    params: VolumeParcelResamplingParameters,
    runner: Runner | None = None,
) -> VolumeParcelResamplingOutputs:
    """
    volume-parcel-resampling
    
    Smooth and resample volume parcels.
    
    Smooths and resamples the region inside each label in cur-parcels to the
    region of the same label name in new-parcels. Any voxels in the output label
    region but outside the input label region will be extrapolated from nearby
    data. The -fix-zeros option causes the smoothing to not use an input value
    if it is zero, but still write a smoothed value to the voxel, and after
    smoothing is complete, it will check for any remaining values of zero, and
    fill them in with extrapolated values.
    
    Note: all volumes must have the same dimensions and spacing. To use a
    different output space, see -volume-parcel-resampling-generic.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeParcelResamplingOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_PARCEL_RESAMPLING_METADATA)
    params = execution.params(params)
    cargs = volume_parcel_resampling_cargs(params, execution)
    ret = volume_parcel_resampling_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_parcel_resampling(
    volume_in: InputPathType,
    cur_parcels: InputPathType,
    new_parcels: InputPathType,
    kernel: float,
    volume_out: str,
    opt_fix_zeros: bool = False,
    opt_fwhm: bool = False,
    opt_subvolume_subvol: str | None = None,
    runner: Runner | None = None,
) -> VolumeParcelResamplingOutputs:
    """
    volume-parcel-resampling
    
    Smooth and resample volume parcels.
    
    Smooths and resamples the region inside each label in cur-parcels to the
    region of the same label name in new-parcels. Any voxels in the output label
    region but outside the input label region will be extrapolated from nearby
    data. The -fix-zeros option causes the smoothing to not use an input value
    if it is zero, but still write a smoothed value to the voxel, and after
    smoothing is complete, it will check for any remaining values of zero, and
    fill them in with extrapolated values.
    
    Note: all volumes must have the same dimensions and spacing. To use a
    different output space, see -volume-parcel-resampling-generic.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        volume_in: the input data volume.
        cur_parcels: label volume of where the parcels currently are.
        new_parcels: label volume of where the parcels should be.
        kernel: gaussian kernel size in mm to smooth by during resampling, as\
            sigma by default.
        volume_out: output volume.
        opt_fix_zeros: treat zero values as not being data.
        opt_fwhm: smoothing kernel size is FWHM, not sigma.
        opt_subvolume_subvol: select a single subvolume as input: the subvolume\
            number or name.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeParcelResamplingOutputs`).
    """
    params = volume_parcel_resampling_params(
        volume_in=volume_in,
        cur_parcels=cur_parcels,
        new_parcels=new_parcels,
        kernel=kernel,
        volume_out=volume_out,
        opt_fix_zeros=opt_fix_zeros,
        opt_fwhm=opt_fwhm,
        opt_subvolume_subvol=opt_subvolume_subvol,
    )
    return volume_parcel_resampling_execute(params, runner)


__all__ = [
    "VOLUME_PARCEL_RESAMPLING_METADATA",
    "VolumeParcelResamplingOutputs",
    "volume_parcel_resampling",
    "volume_parcel_resampling_execute",
    "volume_parcel_resampling_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_REORDER_METADATA = Metadata(
    id="0ad5648e651c6301fc696a16b14ed6ef9efc41a4.boutiques",
    name="cifti-reorder",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


CiftiReorderParameters = typing.TypedDict('CiftiReorderParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/cifti-reorder"]],
    "cifti_in": InputPathType,
    "direction": str,
    "reorder_list": str,
    "cifti_out": str,
})
CiftiReorderParametersTagged = typing.TypedDict('CiftiReorderParametersTagged', {
    "@type": typing.Literal["workbench/cifti-reorder"],
    "cifti_in": InputPathType,
    "direction": str,
    "reorder_list": str,
    "cifti_out": str,
})


class CiftiReorderOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiReorderParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the reordered cifti file"""


def cifti_reorder_params(
    cifti_in: InputPathType,
    direction: str,
    reorder_list: str,
    cifti_out: str,
) -> CiftiReorderParametersTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: input cifti file.
        direction: which dimension to reorder along, ROW or COLUMN.
        reorder_list: a text file containing the desired order transformation.
        cifti_out: the reordered cifti file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-reorder",
        "cifti_in": cifti_in,
        "direction": direction,
        "reorder_list": reorder_list,
        "cifti_out": cifti_out,
    }
    return params


def cifti_reorder_cargs(
    params: CiftiReorderParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-cifti-reorder")
    cargs.append(execution.input_file(params.get("cifti_in", None)))
    cargs.append(params.get("direction", None))
    cargs.append(params.get("reorder_list", None))
    cargs.append(params.get("cifti_out", None))
    return cargs


def cifti_reorder_outputs(
    params: CiftiReorderParameters,
    execution: Execution,
) -> CiftiReorderOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiReorderOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti_out", None)),
    )
    return ret


def cifti_reorder_execute(
    params: CiftiReorderParameters,
    runner: Runner | None = None,
) -> CiftiReorderOutputs:
    """
    cifti-reorder
    
    Reorder the parcels or scalar/label maps in a cifti file.
    
    The mapping along the specified direction must be parcels, scalars, or
    labels. For pscalar or ptseries, use COLUMN to reorder the parcels. For
    dlabel, use ROW. The <reorder-list> file must contain 1-based indices
    separated by whitespace (spaces, newlines, tabs, etc), with as many indices
    as <cifti-in> has along the specified dimension. These indices specify which
    current index should end up in that position, for instance, if the current
    order is 'A B C D', and the desired order is 'D A B C', the text file should
    contain '4 1 2 3'.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiReorderOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_REORDER_METADATA)
    params = execution.params(params)
    cargs = cifti_reorder_cargs(params, execution)
    ret = cifti_reorder_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_reorder(
    cifti_in: InputPathType,
    direction: str,
    reorder_list: str,
    cifti_out: str,
    runner: Runner | None = None,
) -> CiftiReorderOutputs:
    """
    cifti-reorder
    
    Reorder the parcels or scalar/label maps in a cifti file.
    
    The mapping along the specified direction must be parcels, scalars, or
    labels. For pscalar or ptseries, use COLUMN to reorder the parcels. For
    dlabel, use ROW. The <reorder-list> file must contain 1-based indices
    separated by whitespace (spaces, newlines, tabs, etc), with as many indices
    as <cifti-in> has along the specified dimension. These indices specify which
    current index should end up in that position, for instance, if the current
    order is 'A B C D', and the desired order is 'D A B C', the text file should
    contain '4 1 2 3'.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        cifti_in: input cifti file.
        direction: which dimension to reorder along, ROW or COLUMN.
        reorder_list: a text file containing the desired order transformation.
        cifti_out: the reordered cifti file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiReorderOutputs`).
    """
    params = cifti_reorder_params(
        cifti_in=cifti_in,
        direction=direction,
        reorder_list=reorder_list,
        cifti_out=cifti_out,
    )
    return cifti_reorder_execute(params, runner)


__all__ = [
    "CIFTI_REORDER_METADATA",
    "CiftiReorderOutputs",
    "cifti_reorder",
    "cifti_reorder_execute",
    "cifti_reorder_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_ESTIMATE_FWHM_METADATA = Metadata(
    id="1dc8ef5c509e7c0da3d3148fdee937da14d79caf.boutiques",
    name="metric-estimate-fwhm",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


MetricEstimateFwhmWholeFileParameters = typing.TypedDict('MetricEstimateFwhmWholeFileParameters', {
    "@type": typing.NotRequired[typing.Literal["whole_file"]],
    "opt_demean": bool,
})
MetricEstimateFwhmWholeFileParametersTagged = typing.TypedDict('MetricEstimateFwhmWholeFileParametersTagged', {
    "@type": typing.Literal["whole_file"],
    "opt_demean": bool,
})


MetricEstimateFwhmParameters = typing.TypedDict('MetricEstimateFwhmParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/metric-estimate-fwhm"]],
    "surface": InputPathType,
    "metric_in": InputPathType,
    "opt_roi_roi_metric": typing.NotRequired[InputPathType | None],
    "opt_column_column": typing.NotRequired[str | None],
    "whole_file": typing.NotRequired[MetricEstimateFwhmWholeFileParameters | None],
})
MetricEstimateFwhmParametersTagged = typing.TypedDict('MetricEstimateFwhmParametersTagged', {
    "@type": typing.Literal["workbench/metric-estimate-fwhm"],
    "surface": InputPathType,
    "metric_in": InputPathType,
    "opt_roi_roi_metric": typing.NotRequired[InputPathType | None],
    "opt_column_column": typing.NotRequired[str | None],
    "whole_file": typing.NotRequired[MetricEstimateFwhmWholeFileParameters | None],
})


def metric_estimate_fwhm_whole_file_params(
    opt_demean: bool = False,
) -> MetricEstimateFwhmWholeFileParametersTagged:
    """
    Build parameters.
    
    Args:
        opt_demean: subtract the mean image before estimating smoothness.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "whole_file",
        "opt_demean": opt_demean,
    }
    return params


def metric_estimate_fwhm_whole_file_cargs(
    params: MetricEstimateFwhmWholeFileParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-whole-file")
    if params.get("opt_demean", False):
        cargs.append("-demean")
    return cargs


class MetricEstimateFwhmOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricEstimateFwhmParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def metric_estimate_fwhm_params(
    surface: InputPathType,
    metric_in: InputPathType,
    opt_roi_roi_metric: InputPathType | None = None,
    opt_column_column: str | None = None,
    whole_file: MetricEstimateFwhmWholeFileParameters | None = None,
) -> MetricEstimateFwhmParametersTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface to use for distance and neighbor information.
        metric_in: the input metric.
        opt_roi_roi_metric: use only data within an ROI: the metric file to use\
            as an ROI.
        opt_column_column: select a single column to estimate smoothness of:\
            the column number or name.
        whole_file: estimate for the whole file at once, not each column\
            separately.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-estimate-fwhm",
        "surface": surface,
        "metric_in": metric_in,
    }
    if opt_roi_roi_metric is not None:
        params["opt_roi_roi_metric"] = opt_roi_roi_metric
    if opt_column_column is not None:
        params["opt_column_column"] = opt_column_column
    if whole_file is not None:
        params["whole_file"] = whole_file
    return params


def metric_estimate_fwhm_cargs(
    params: MetricEstimateFwhmParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-metric-estimate-fwhm")
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(execution.input_file(params.get("metric_in", None)))
    if params.get("opt_roi_roi_metric", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("opt_roi_roi_metric", None))
        ])
    if params.get("opt_column_column", None) is not None:
        cargs.extend([
            "-column",
            params.get("opt_column_column", None)
        ])
    if params.get("whole_file", None) is not None:
        cargs.extend(metric_estimate_fwhm_whole_file_cargs(params.get("whole_file", None), execution))
    return cargs


def metric_estimate_fwhm_outputs(
    params: MetricEstimateFwhmParameters,
    execution: Execution,
) -> MetricEstimateFwhmOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricEstimateFwhmOutputs(
        root=execution.output_file("."),
    )
    return ret


def metric_estimate_fwhm_execute(
    params: MetricEstimateFwhmParameters,
    runner: Runner | None = None,
) -> MetricEstimateFwhmOutputs:
    """
    metric-estimate-fwhm
    
    Estimate fwhm smoothness of a metric file.
    
    Estimates the smoothness of the metric columns, printing the estimates to
    standard output. These estimates ignore variation in vertex spacing.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricEstimateFwhmOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_ESTIMATE_FWHM_METADATA)
    params = execution.params(params)
    cargs = metric_estimate_fwhm_cargs(params, execution)
    ret = metric_estimate_fwhm_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_estimate_fwhm(
    surface: InputPathType,
    metric_in: InputPathType,
    opt_roi_roi_metric: InputPathType | None = None,
    opt_column_column: str | None = None,
    whole_file: MetricEstimateFwhmWholeFileParameters | None = None,
    runner: Runner | None = None,
) -> MetricEstimateFwhmOutputs:
    """
    metric-estimate-fwhm
    
    Estimate fwhm smoothness of a metric file.
    
    Estimates the smoothness of the metric columns, printing the estimates to
    standard output. These estimates ignore variation in vertex spacing.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        surface: the surface to use for distance and neighbor information.
        metric_in: the input metric.
        opt_roi_roi_metric: use only data within an ROI: the metric file to use\
            as an ROI.
        opt_column_column: select a single column to estimate smoothness of:\
            the column number or name.
        whole_file: estimate for the whole file at once, not each column\
            separately.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricEstimateFwhmOutputs`).
    """
    params = metric_estimate_fwhm_params(
        surface=surface,
        metric_in=metric_in,
        opt_roi_roi_metric=opt_roi_roi_metric,
        opt_column_column=opt_column_column,
        whole_file=whole_file,
    )
    return metric_estimate_fwhm_execute(params, runner)


__all__ = [
    "METRIC_ESTIMATE_FWHM_METADATA",
    "MetricEstimateFwhmOutputs",
    "metric_estimate_fwhm",
    "metric_estimate_fwhm_execute",
    "metric_estimate_fwhm_params",
    "metric_estimate_fwhm_whole_file_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_MERGE_METADATA = Metadata(
    id="0484245a1d35b7ed32a01a9b8363c20b753b6141.boutiques",
    name="volume-merge",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


VolumeMergeUpToParameters = typing.TypedDict('VolumeMergeUpToParameters', {
    "@type": typing.NotRequired[typing.Literal["up_to"]],
    "last_subvol": str,
    "opt_reverse": bool,
})
VolumeMergeUpToParametersTagged = typing.TypedDict('VolumeMergeUpToParametersTagged', {
    "@type": typing.Literal["up_to"],
    "last_subvol": str,
    "opt_reverse": bool,
})


VolumeMergeSubvolumeParameters = typing.TypedDict('VolumeMergeSubvolumeParameters', {
    "@type": typing.NotRequired[typing.Literal["subvolume"]],
    "subvol": str,
    "up_to": typing.NotRequired[VolumeMergeUpToParameters | None],
})
VolumeMergeSubvolumeParametersTagged = typing.TypedDict('VolumeMergeSubvolumeParametersTagged', {
    "@type": typing.Literal["subvolume"],
    "subvol": str,
    "up_to": typing.NotRequired[VolumeMergeUpToParameters | None],
})


VolumeMergeVolumeParameters = typing.TypedDict('VolumeMergeVolumeParameters', {
    "@type": typing.NotRequired[typing.Literal["volume"]],
    "volume_in": InputPathType,
    "subvolume": typing.NotRequired[list[VolumeMergeSubvolumeParameters] | None],
})
VolumeMergeVolumeParametersTagged = typing.TypedDict('VolumeMergeVolumeParametersTagged', {
    "@type": typing.Literal["volume"],
    "volume_in": InputPathType,
    "subvolume": typing.NotRequired[list[VolumeMergeSubvolumeParameters] | None],
})


VolumeMergeParameters = typing.TypedDict('VolumeMergeParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/volume-merge"]],
    "volume_out": str,
    "volume": typing.NotRequired[list[VolumeMergeVolumeParameters] | None],
})
VolumeMergeParametersTagged = typing.TypedDict('VolumeMergeParametersTagged', {
    "@type": typing.Literal["workbench/volume-merge"],
    "volume_out": str,
    "volume": typing.NotRequired[list[VolumeMergeVolumeParameters] | None],
})


def volume_merge_up_to_params(
    last_subvol: str,
    opt_reverse: bool = False,
) -> VolumeMergeUpToParametersTagged:
    """
    Build parameters.
    
    Args:
        last_subvol: the number or name of the last subvolume to include.
        opt_reverse: use the range in reverse order.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "up_to",
        "last_subvol": last_subvol,
        "opt_reverse": opt_reverse,
    }
    return params


def volume_merge_up_to_cargs(
    params: VolumeMergeUpToParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-up-to")
    cargs.append(params.get("last_subvol", None))
    if params.get("opt_reverse", False):
        cargs.append("-reverse")
    return cargs


def volume_merge_subvolume_params(
    subvol: str,
    up_to: VolumeMergeUpToParameters | None = None,
) -> VolumeMergeSubvolumeParametersTagged:
    """
    Build parameters.
    
    Args:
        subvol: the subvolume number or name.
        up_to: use an inclusive range of subvolumes.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "subvolume",
        "subvol": subvol,
    }
    if up_to is not None:
        params["up_to"] = up_to
    return params


def volume_merge_subvolume_cargs(
    params: VolumeMergeSubvolumeParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-subvolume")
    cargs.append(params.get("subvol", None))
    if params.get("up_to", None) is not None:
        cargs.extend(volume_merge_up_to_cargs(params.get("up_to", None), execution))
    return cargs


def volume_merge_volume_params(
    volume_in: InputPathType,
    subvolume: list[VolumeMergeSubvolumeParameters] | None = None,
) -> VolumeMergeVolumeParametersTagged:
    """
    Build parameters.
    
    Args:
        volume_in: a volume file to use subvolumes from.
        subvolume: select a single subvolume to use.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "volume",
        "volume_in": volume_in,
    }
    if subvolume is not None:
        params["subvolume"] = subvolume
    return params


def volume_merge_volume_cargs(
    params: VolumeMergeVolumeParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-volume")
    cargs.append(execution.input_file(params.get("volume_in", None)))
    if params.get("subvolume", None) is not None:
        cargs.extend([a for c in [volume_merge_subvolume_cargs(s, execution) for s in params.get("subvolume", None)] for a in c])
    return cargs


class VolumeMergeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeMergeParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume file"""


def volume_merge_params(
    volume_out: str,
    volume: list[VolumeMergeVolumeParameters] | None = None,
) -> VolumeMergeParametersTagged:
    """
    Build parameters.
    
    Args:
        volume_out: the output volume file.
        volume: specify an input volume file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-merge",
        "volume_out": volume_out,
    }
    if volume is not None:
        params["volume"] = volume
    return params


def volume_merge_cargs(
    params: VolumeMergeParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-volume-merge")
    cargs.append(params.get("volume_out", None))
    if params.get("volume", None) is not None:
        cargs.extend([a for c in [volume_merge_volume_cargs(s, execution) for s in params.get("volume", None)] for a in c])
    return cargs


def volume_merge_outputs(
    params: VolumeMergeParameters,
    execution: Execution,
) -> VolumeMergeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeMergeOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume_out", None)),
    )
    return ret


def volume_merge_execute(
    params: VolumeMergeParameters,
    runner: Runner | None = None,
) -> VolumeMergeOutputs:
    """
    volume-merge
    
    Merge volume files into a new file.
    
    Takes one or more volume files and constructs a new volume file by
    concatenating subvolumes from them. The input volume files must have the
    same volume space.
    
    Example: wb_command -volume-merge out.nii -volume first.nii -subvolume 1
    -volume second.nii
    
    This example would take the first subvolume from first.nii, followed by all
    subvolumes from second.nii, and write these to out.nii.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeMergeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_MERGE_METADATA)
    params = execution.params(params)
    cargs = volume_merge_cargs(params, execution)
    ret = volume_merge_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_merge(
    volume_out: str,
    volume: list[VolumeMergeVolumeParameters] | None = None,
    runner: Runner | None = None,
) -> VolumeMergeOutputs:
    """
    volume-merge
    
    Merge volume files into a new file.
    
    Takes one or more volume files and constructs a new volume file by
    concatenating subvolumes from them. The input volume files must have the
    same volume space.
    
    Example: wb_command -volume-merge out.nii -volume first.nii -subvolume 1
    -volume second.nii
    
    This example would take the first subvolume from first.nii, followed by all
    subvolumes from second.nii, and write these to out.nii.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        volume_out: the output volume file.
        volume: specify an input volume file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeMergeOutputs`).
    """
    params = volume_merge_params(
        volume_out=volume_out,
        volume=volume,
    )
    return volume_merge_execute(params, runner)


__all__ = [
    "VOLUME_MERGE_METADATA",
    "VolumeMergeOutputs",
    "volume_merge",
    "volume_merge_execute",
    "volume_merge_params",
    "volume_merge_subvolume_params",
    "volume_merge_up_to_params",
    "volume_merge_volume_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

FOCI_CREATE_METADATA = Metadata(
    id="2e6c039919782b63c07e5badc35659f838c016d5.boutiques",
    name="foci-create",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


FociCreateClassParameters = typing.TypedDict('FociCreateClassParameters', {
    "@type": typing.NotRequired[typing.Literal["class"]],
    "class_name": str,
    "foci_list_file": str,
    "surface": InputPathType,
})
FociCreateClassParametersTagged = typing.TypedDict('FociCreateClassParametersTagged', {
    "@type": typing.Literal["class"],
    "class_name": str,
    "foci_list_file": str,
    "surface": InputPathType,
})


FociCreateParameters = typing.TypedDict('FociCreateParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/foci-create"]],
    "output": str,
    "class": typing.NotRequired[list[FociCreateClassParameters] | None],
})
FociCreateParametersTagged = typing.TypedDict('FociCreateParametersTagged', {
    "@type": typing.Literal["workbench/foci-create"],
    "output": str,
    "class": typing.NotRequired[list[FociCreateClassParameters] | None],
})


def foci_create_class_params(
    class_name: str,
    foci_list_file: str,
    surface: InputPathType,
) -> FociCreateClassParametersTagged:
    """
    Build parameters.
    
    Args:
        class_name: name of class.
        foci_list_file: text file containing foci names, coordinates, and\
            colors.
        surface: surface file for projection of foci list file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "class",
        "class_name": class_name,
        "foci_list_file": foci_list_file,
        "surface": surface,
    }
    return params


def foci_create_class_cargs(
    params: FociCreateClassParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-class")
    cargs.append(params.get("class_name", None))
    cargs.append(params.get("foci_list_file", None))
    cargs.append(execution.input_file(params.get("surface", None)))
    return cargs


class FociCreateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `FociCreateParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output foci file"""


def foci_create_params(
    output: str,
    class_: list[FociCreateClassParameters] | None = None,
) -> FociCreateParametersTagged:
    """
    Build parameters.
    
    Args:
        output: the output foci file.
        class_: specify class input data.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/foci-create",
        "output": output,
    }
    if class_ is not None:
        params["class"] = class_
    return params


def foci_create_cargs(
    params: FociCreateParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-foci-create")
    cargs.append(params.get("output", None))
    if params.get("class", None) is not None:
        cargs.extend([a for c in [foci_create_class_cargs(s, execution) for s in params.get("class", None)] for a in c])
    return cargs


def foci_create_outputs(
    params: FociCreateParameters,
    execution: Execution,
) -> FociCreateOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = FociCreateOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def foci_create_execute(
    params: FociCreateParameters,
    runner: Runner | None = None,
) -> FociCreateOutputs:
    """
    foci-create
    
    Create a foci file.
    
    Creates a foci file from names, coordinates, and RGB values in a text file.
    The text file must have the following format (2 lines per focus):
    
    <focus-name>
    <red> <green> <blue> <x> <y> <z>
    ...
    
    Foci names are specified on a separate line from their coordinates and
    color, in order to let foci names contain spaces. Whitespace is trimmed from
    both ends of the foci name, but is kept if it is in the middle of a name.
    The values of <red>, <green>, <blue> and must be integers from 0 to 255, and
    will specify the color the foci is drawn as.
    
    Foci are grouped into classes and the name for the class is specified using
    the <class-name> parameter.
    
    All foci within one text file must be associated with the structure
    contained in the <surface> parameter and are projected to that surface.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FociCreateOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FOCI_CREATE_METADATA)
    params = execution.params(params)
    cargs = foci_create_cargs(params, execution)
    ret = foci_create_outputs(params, execution)
    execution.run(cargs)
    return ret


def foci_create(
    output: str,
    class_: list[FociCreateClassParameters] | None = None,
    runner: Runner | None = None,
) -> FociCreateOutputs:
    """
    foci-create
    
    Create a foci file.
    
    Creates a foci file from names, coordinates, and RGB values in a text file.
    The text file must have the following format (2 lines per focus):
    
    <focus-name>
    <red> <green> <blue> <x> <y> <z>
    ...
    
    Foci names are specified on a separate line from their coordinates and
    color, in order to let foci names contain spaces. Whitespace is trimmed from
    both ends of the foci name, but is kept if it is in the middle of a name.
    The values of <red>, <green>, <blue> and must be integers from 0 to 255, and
    will specify the color the foci is drawn as.
    
    Foci are grouped into classes and the name for the class is specified using
    the <class-name> parameter.
    
    All foci within one text file must be associated with the structure
    contained in the <surface> parameter and are projected to that surface.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        output: the output foci file.
        class_: specify class input data.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FociCreateOutputs`).
    """
    params = foci_create_params(
        output=output,
        class_=class_,
    )
    return foci_create_execute(params, runner)


__all__ = [
    "FOCI_CREATE_METADATA",
    "FociCreateOutputs",
    "foci_create",
    "foci_create_class_params",
    "foci_create_execute",
    "foci_create_params",
]
